from __future__ import annotations

import os
import pathlib
from typing import Literal, Union

from prettyqt import core, gui
from prettyqt.qt import QtCore, QtQuick
from prettyqt.utils import bidict


RESIZE_MODES = bidict(
    view_to_root_object=QtQuick.QQuickView.SizeViewToRootObject,
    root_object_to_view=QtQuick.QQuickView.SizeRootObjectToView,
)

ResizeModeStr = Literal["view_to_root_object", "root_object_to_view"]

STATUS = bidict(
    null=QtQuick.QQuickView.Null,
    ready=QtQuick.QQuickView.Ready,
    loading=QtQuick.QQuickView.Loading,
    error=QtQuick.QQuickView.Error,
)

StatusStr = Literal["null", "ready", "loading", "error"]

QtQuick.QQuickView.__bases__ = (gui.Window,)


class QuickView(QtQuick.QQuickView):
    def set_source(self, source: Union[str, os.PathLike, QtCore.QUrl]):
        if isinstance(source, os.PathLike):  # type: ignore
            source = os.fspath(source)
        if isinstance(source, str):
            source = core.Url.fromLocalFile(source)
        self.setSource(source)

    def get_source(self) -> pathlib.Path:
        return pathlib.Path(self.source().toLocalFile())

    def get_status(self) -> StatusStr:
        return STATUS.inverse[self.status()]



from __future__ import annotations

from prettyqt import gui
from prettyqt.utils import types


WINDOW_ICON_COLOR = "darkcyan"


def set_window_icon_color(color):
    global WINDOW_ICON_COLOR
    WINDOW_ICON_COLOR = color


def get_color(color: types.ColorType) -> gui.Color:
    """Get gui.Color instance for given parameter.

    named colors are 'aliceblue', 'antiquewhite', 'aqua', 'aquamarine',
    'azure', 'beige', 'bisque', 'black', 'blanchedalmond', 'blue', 'blueviolet',
    'brown', 'burlywood', 'cadetblue', 'chartreuse', 'chocolate', 'coral',
    'cornflowerblue', 'cornsilk', 'crimson', 'cyan', 'darkblue', 'darkcyan',
    'darkgoldenrod', 'darkgray', 'darkgreen', 'darkgrey', 'darkkhaki',
    'darkmagenta', 'darkolivegreen', 'darkorange', 'darkorchid', 'darkred',
    'darksalmon', 'darkseagreen', 'darkslateblue', 'darkslategray', 'darkslategrey',
    'darkturquoise', 'darkviolet', 'deeppink', 'deepskyblue', 'dimgray', 'dimgrey',
    'dodgerblue', 'firebrick', 'floralwhite', 'forestgreen', 'fuchsia', 'gainsboro',
    'ghostwhite', 'gold', 'goldenrod', 'gray', 'green', 'greenyellow', 'grey',
    'honeydew', 'hotpink', 'indianred', 'indigo', 'ivory', 'khaki', 'lavender',
    'lavenderblush', 'lawngreen', 'lemonchiffon', 'lightblue', 'lightcoral',
    'lightcyan', 'lightgoldenrodyellow', 'lightgray', 'lightgreen', 'lightgrey',
    'lightpink', 'lightsalmon', 'lightseagreen', 'lightskyblue', 'lightslategray',
    'lightslategrey', 'lightsteelblue', 'lightyellow', 'lime', 'limegreen', 'linen',
    'magenta', 'maroon', 'mediumaquamarine', 'mediumblue', 'mediumorchid',
    'mediumpurple', 'mediumseagreen', 'mediumslateblue', 'mediumspringgreen',
    'mediumturquoise', 'mediumvioletred', 'midnightblue', 'mintcream', 'mistyrose',
    'moccasin', 'navajowhite', 'navy', 'oldlace', 'olive', 'olivedrab', 'orange',
    'orangered', 'orchid', 'palegoldenrod', 'palegreen', 'paleturquoise',
    'palevioletred', 'papayawhip', 'peachpuff', 'peru', 'pink', 'plum',
    'powderblue', 'purple', 'red', 'rosybrown', 'royalblue', 'saddlebrown',
    'salmon', 'sandybrown', 'seagreen', 'seashell', 'sienna', 'silver', 'skyblue',
    'slateblue', 'slategray', 'slategrey', 'snow', 'springgreen', 'steelblue',
    'tan', 'teal', 'thistle', 'tomato', 'transparent', 'turquoise', 'violet',
    'wheat', 'white', 'whitesmoke', 'yellow', 'yellowgreen'

    Args:
        color (ColorType): color to create gui.Icon from

    Returns:
        gui.Color: color instance
    """
    if isinstance(color, (tuple, list)):
        return gui.Color(*color)
    return gui.Color(color)



from __future__ import annotations

from typing import Tuple, Union

from prettyqt.qt import QtCore


class Size(QtCore.QSize):
    def __repr__(self):
        return f"{type(self).__name__}({self.width()}, {self.height()})"

    def __getitem__(self, index) -> int:
        return (self.width(), self.height())[index]

    def __reduce__(self):
        return type(self), (self.width(), self.height())

    def expanded_to(self, size: Union[QtCore.QSize, Tuple[int, int]]) -> Size:
        if isinstance(size, tuple):
            size = QtCore.QSize(*size)
        return Size(self.expandedTo(size))


if __name__ == "__main__":
    size = Size(10, 20)
    print(tuple(size))
    size = size.expanded_to(QtCore.QSize(100, 100))
    print(type(size))



from __future__ import annotations

from typing import List, Literal

from prettyqt import core, gui, widgets
from prettyqt.qt import QtGui, QtWidgets
from prettyqt.utils import InvalidParamError, bidict, helpers


FONT_FILTERS = bidict(
    all=QtWidgets.QFontComboBox.AllFonts,
    scalable=QtWidgets.QFontComboBox.ScalableFonts,
    non_scalable=QtWidgets.QFontComboBox.NonScalableFonts,
    monospaced=QtWidgets.QFontComboBox.MonospacedFonts,
    proportional=QtWidgets.QFontComboBox.ProportionalFonts,
)

FontFilterStr = Literal["all", "scalable", "non_scalable", "monospaced", "proportional"]


QtWidgets.QFontComboBox.__bases__ = (widgets.ComboBox,)


class FontComboBox(QtWidgets.QFontComboBox):

    value_changed = core.Signal(object)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.currentIndexChanged.connect(self.index_changed)

    def serialize_fields(self):
        return dict(
            current_font=self.get_current_font(),
            font_filters=self.get_font_filters(),
        )

    def __setstate__(self, state):
        self.set_font_filters(*state.get("font_filters", []))
        self.setCurrentFont(state["current_font"])

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def set_font_filters(self, *filters: FontFilterStr):
        """Set font filters.

        Args:
            filters: font filters to use

        Raises:
            InvalidParamError: invalid font filters
        """
        if not filters:
            filters = ("all",)
        for item in filters:
            if item not in FONT_FILTERS:
                raise InvalidParamError(item, FONT_FILTERS)
        flags = helpers.merge_flags(filters, FONT_FILTERS)
        self.setFontFilters(flags)

    def get_font_filters(self) -> List[FontFilterStr]:
        """Return list of font filters.

        Returns:
            font filter list
        """
        return [k for k, v in FONT_FILTERS.items() if v & self.fontFilters()]

    def set_value(self, value: QtGui.QFont):
        self.setCurrentFont(value)

    def get_value(self) -> gui.Font:
        return self.get_current_font()

    def get_current_font(self) -> gui.Font:
        return gui.Font(self.currentFont())


if __name__ == "__main__":
    app = widgets.app()
    widget = FontComboBox()
    widget.value_changed.connect(print)
    widget.set_value(gui.Font("Script"))
    widget.show()
    app.main_loop()



from __future__ import annotations

import contextlib
from typing import Literal

from prettyqt import core
from prettyqt.qt import QtCore
from prettyqt.utils import InvalidParamError, mappers


if core.VersionNumber.get_qt_version() >= (6, 0, 0):
    mod = QtCore.QIODeviceBase  # type: ignore
    base = (core.Object, QtCore.QIODeviceBase)  # type: ignore
else:
    mod = QtCore.QIODevice  # type: ignore
    base = (core.Object,)  # type: ignore

OPEN_MODES = mappers.FlagMap(
    mod.OpenModeFlag,
    not_open=mod.NotOpen,
    read_only=mod.ReadOnly,
    write_only=mod.WriteOnly,
    read_write=mod.ReadWrite,
    append=mod.Append,
    truncate=mod.Truncate,
    text=mod.Text,
    unbuffered=mod.Unbuffered,
    new_only=mod.NewOnly,
    existing_only=mod.ExistingOnly,
)

OpenModeStr = Literal[
    "not_open",
    "read_only",
    "write_only",
    "read_write",
    "append",
    "truncate",
    "text",
    "unbuffered",
    "new_only",
    "existing_only",
]

QtCore.QIODevice.__bases__ = base


class IODevice(QtCore.QIODevice):
    @contextlib.contextmanager
    def open_file(self, mode: OpenModeStr):
        if mode not in OPEN_MODES:
            raise InvalidParamError(mode, OPEN_MODES)
        self.open(OPEN_MODES[mode])
        yield self
        self.close()

    def get_open_mode(self) -> OpenModeStr:
        return OPEN_MODES.inverse[self.openMode()]



from __future__ import annotations

from typing import Literal, Tuple, Union

from prettyqt import core
from prettyqt.qt import QtCore, QtGui
from prettyqt.utils import InvalidParamError, bidict


CURSOR_POSITION = bidict(
    cursor_between_characters=QtGui.QTextLine.CursorBetweenCharacters,
    cursor_on_character=QtGui.QTextLine.CursorOnCharacter,
)

CursorPositionStr = Literal["cursor_between_characters", "cursor_on_character"]

EDGE = bidict(
    leading=QtGui.QTextLine.Leading,
    trailing=QtGui.QTextLine.Trailing,
)

EdgeStr = Literal["leading", "trailing"]


class TextLine(QtGui.QTextLine):
    def __bool__(self):
        return self.isValid()

    def __repr__(self):
        return f"{type(self).__name__}()"

    def __len__(self):
        return self.textLength()

    def get_position(self) -> core.Point:
        return core.Point(self.position())

    def set_position(self, point: Union[QtCore.QPoint, Tuple[int, int]]):
        if isinstance(point, tuple):
            point = core.Point(*point)
        self.setPosition(point)

    def cursor_to_x(self, cursor_pos: int, edge: EdgeStr = "leading") -> float:
        if edge not in EDGE:
            raise InvalidParamError(edge, EDGE)
        return self.cursorToX(cursor_pos, EDGE[edge])

    def x_to_cursor(
        self, x: float, cursor_pos: CursorPositionStr = "cursor_between_characters"
    ) -> int:
        if cursor_pos not in CURSOR_POSITION:
            raise InvalidParamError(cursor_pos, CURSOR_POSITION)
        return self.xToCursor(x, CURSOR_POSITION[cursor_pos])


if __name__ == "__main__":
    fmt = TextLine()
    print(bool(fmt))



from __future__ import annotations

from prettyqt.qt import QtCore


class MimeType(QtCore.QMimeType):
    def __bool__(self):
        return self.isValid()

    def __str__(self):
        return self.name()



from __future__ import annotations

import contextlib
from typing import List, Optional, Tuple, Union

from prettyqt import constants, core, gui, iconprovider
from prettyqt.qt import QtCore, QtGui
from prettyqt.utils import InvalidParamError, colors, types


QtGui.QGuiApplication.__bases__ = (core.CoreApplication,)


class GuiApplication(QtGui.QGuiApplication):
    def serialize_fields(self):
        return dict(icon=self.get_icon())

    @classmethod
    @contextlib.contextmanager
    def override_cursor(cls, cursor: constants.CursorShapeStr):
        cls.set_override_cursor(cursor)
        yield cursor
        cls.restore_override_cursor()

    @classmethod
    def set_override_cursor(cls, cursor: constants.CursorShapeStr):
        cursor = gui.Cursor(constants.CURSOR_SHAPE[cursor])
        cls.setOverrideCursor(cursor)

    @classmethod
    def restore_override_cursor(cls):
        cls.restoreOverrideCursor()

    @classmethod
    def get_clipboard(cls) -> gui.Clipboard:
        return gui.Clipboard(cls.clipboard())

    def set_layout_direction(self, direction: constants.LayoutDirectionStr):
        """Set layout direction.

        Args:
            direction: layout direction

        Raises:
            InvalidParamError: layout direction does not exist
        """
        if direction not in constants.LAYOUT_DIRECTION:
            raise InvalidParamError(direction, constants.LAYOUT_DIRECTION)
        self.setLayoutDirection(constants.LAYOUT_DIRECTION[direction])

    def get_layout_direction(self) -> constants.LayoutDirectionStr:
        """Get the current layout direction.

        Returns:
            layout direction
        """
        return constants.LAYOUT_DIRECTION.inverse[self.layoutDirection()]

    @classmethod
    def set_high_dpi_scale_factor_rounding_policy(
        cls, policy: constants.HighDpiScaleFactorRoundingPolicyStr
    ):
        """Set high dpi scale factor rounding policy.

        Args:
            policy: rounding policy

        Raises:
            InvalidParamError: rounding policy does not exist
        """
        if policy not in constants.HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY:
            raise InvalidParamError(
                policy, constants.HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY
            )
        cls.setHighDpiScaleFactorRoundingPolicy(
            constants.HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY[policy]
        )

    @classmethod
    def get_high_dpi_scale_factor_rounding_policy(
        cls,
    ) -> constants.HighDpiScaleFactorRoundingPolicyStr:
        """Get the current high dpi scale factor rounding policy.

        Returns:
            rounding policy
        """
        return constants.HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY.inverse[
            cls.highDpiScaleFactorRoundingPolicy()
        ]

    @classmethod
    def get_application_state(cls) -> List[constants.ApplicationStateStr]:
        """Get the current application state.

        Returns:
            application state
        """
        return [
            k
            for k, v in constants.APPLICATION_STATES.items()
            if v & cls.applicationState()  # type: ignore
        ]

    def get_primary_screen(self) -> gui.Screen:
        return gui.Screen(self.primaryScreen())

    def get_screen_at(self, point: Union[Tuple[int, int], QtCore.QPoint]) -> gui.Screen:
        if isinstance(point, tuple):
            point = QtCore.QPoint(*point)
        return gui.Screen(self.screenAt(point))

    def get_screens(self) -> List[gui.Screen]:
        return [gui.Screen(i) for i in self.screens()]

    @classmethod
    def get_input_method(cls) -> gui.InputMethod:
        return gui.InputMethod(cls.inputMethod())

    @classmethod
    def copy_to_clipboard(cls, text: str):
        """Sets clipboard to supplied text."""
        cb = cls.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText(text, mode=cb.Clipboard)

    @classmethod
    def get_font(cls) -> gui.Font:
        return gui.Font(cls.font())

    def set_icon(self, icon: types.IconType):
        """Set the default window icon.

        Args:
            icon: icon to use
        """
        icon = iconprovider.get_icon(icon, color=colors.WINDOW_ICON_COLOR)
        self.setWindowIcon(icon)

    def get_icon(self) -> Optional[gui.Icon]:
        icon = self.windowIcon()
        if icon.isNull():
            return None
        return gui.Icon(self.windowIcon())

    @classmethod
    def get_palette(cls) -> gui.Palette:
        return gui.Palette(cls.palette())



from __future__ import annotations

import sys

from prettyqt import core, network, widgets


class SingleApplication(widgets.Application):

    messageReceived = core.Signal(str)

    def __init__(self, app_id: str):

        super().__init__(sys.argv)
        self.app_id = app_id
        self._activate_on_message = True

        # Is there another instance running?
        out_socket = network.LocalSocket()
        out_socket.connect_to_server(self.app_id)
        self._is_running = out_socket.waitForConnected()
        self._in_socket = None
        self._in_stream = None
        if not self._is_running:
            self._out_socket = None
            self._out_stream = None
            self._server = network.LocalServer()
            self._server.listen(self.app_id)
            self._server.newConnection.connect(self._on_new_connection)
        else:
            self._out_socket = out_socket
            self._out_stream = core.TextStream(self._out_socket)
            self._out_stream.set_codec("UTF-8")

    def is_running(self) -> bool:
        return self._is_running

    def activate_window(self):
        window = self.get_mainwindow()
        if window is None:
            return
        window.raise_to_top()

    def send_message(self, msg: str) -> bool:
        if self._out_stream is None or self._out_socket is None:
            return False
        self._out_stream << msg << "\n"
        self._out_stream.flush()
        return self._out_socket.waitForBytesWritten()

    def _on_new_connection(self):
        if self._in_socket:
            self._in_socket.readyRead.disconnect(self._on_ready_read)
        self._in_socket = self._server.nextPendingConnection()  # type: ignore
        if self._in_socket is None:
            return
        self._in_stream = core.TextStream(self._in_socket)
        self._in_stream.set_codec("UTF-8")
        self._in_socket.readyRead.connect(self._on_ready_read)
        if self._activate_on_message:
            self.activate_window()

    def _on_ready_read(self):
        if self._in_stream is None:
            raise RuntimeError()
        for msg in self._in_stream.read_lines():
            self.messageReceived.emit(msg)


if __name__ == "__main__":
    app1 = SingleApplication("test")
    print(app1.is_running())
    app1.exec()



from __future__ import annotations

import os
from typing import Iterator, Literal, Optional, Union

from prettyqt import iconprovider
from prettyqt.qt import QtWinExtras
from prettyqt.utils import bidict, types


TYPES = bidict(
    custom=QtWinExtras.QWinJumpListCategory.Custom,
    recent=QtWinExtras.QWinJumpListCategory.Recent,
    frequent=QtWinExtras.QWinJumpListCategory.Frequent,
    tasks=QtWinExtras.QWinJumpListCategory.Tasks,
)

TypeStr = Literal["custom", "recent", "frequent", "tasks"]


class WinJumpListCategory(QtWinExtras.QWinJumpListCategory):
    def __contains__(self, item):
        return item in self.items()

    def __len__(self) -> int:
        return self.count()

    def __bool__(self):
        return not self.isEmpty()

    def __iter__(self) -> Iterator[QtWinExtras.QWinJumpListItem]:
        return iter(self.items())

    def get_type(self) -> TypeStr:
        return TYPES.inverse[self.type()]

    def add_destination(self, destination: Union[str, os.PathLike]) -> None:
        self.addDestination(os.fspath(destination))

    def set_title(self, title: str) -> None:
        self.setTitle(title)

    def add_link(
        self,
        title: str,
        exe_path: Union[str, os.PathLike],
        arguments: Optional[list] = None,
        icon: types.IconType = None,
    ) -> None:
        icon = iconprovider.get_icon(icon)
        if arguments is None:
            arguments = []
        self.addLink(icon, title, os.fspath(exe_path), arguments)


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    cat = WinJumpListCategory()
    len(cat)
    app.main_loop()



from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


QtWidgets.QStyleOptionButton.__bases__ = (widgets.StyleOption,)


class StyleOptionButton(QtWidgets.QStyleOptionButton):
    pass



from __future__ import annotations

from typing import Literal

from prettyqt import constants, core
from prettyqt.qt import QtWidgets
from prettyqt.utils import InvalidParamError, bidict


SUBMIT_POLICY = bidict(
    auto=QtWidgets.QDataWidgetMapper.AutoSubmit,
    manual=QtWidgets.QDataWidgetMapper.ManualSubmit,
)

SubmitPolicyStr = Literal["auto", "manual"]

QtWidgets.QDataWidgetMapper.__bases__ = (core.Object,)


class DataWidgetMapper(QtWidgets.QDataWidgetMapper):
    def serialize_fields(self):
        return dict(
            current_index=self.currentIndex(),
            orientation=self.get_orientation(),
            submit_policy=self.get_submit_policy(),
        )

    def __setstate__(self, state):
        super().__setstate__(state)
        self.set_orientation(state["orientation"])
        self.set_submit_policy(state["submit_policy"])
        self.setCurrentIndex(state["current_index"])

    def set_orientation(self, orientation: constants.OrientationStr):
        """Set the orientation of the data widget mapper.

        Args:
            orientation: orientation for the data widget mapper

        Raises:
            InvalidParamError: orientation does not exist
        """
        if orientation not in constants.ORIENTATION:
            raise InvalidParamError(orientation, constants.ORIENTATION)
        self.setOrientation(constants.ORIENTATION[orientation])

    def get_orientation(self) -> constants.OrientationStr:
        """Return current orientation.

        Returns:
            orientation
        """
        return constants.ORIENTATION.inverse[self.orientation()]

    def set_submit_policy(self, policy: SubmitPolicyStr):
        """Set the submit policy of the mapper.

        Args:
            policy: submit_policy for the data widget mapper

        Raises:
            InvalidParamError: submit_policy does not exist
        """
        if policy not in SUBMIT_POLICY:
            raise InvalidParamError(policy, SUBMIT_POLICY)
        self.setSubmitPolicy(SUBMIT_POLICY[policy])

    def get_submit_policy(self) -> SubmitPolicyStr:
        """Return current submit policy.

        Returns:
            submit policy
        """
        return SUBMIT_POLICY.inverse[self.submitPolicy()]



from __future__ import annotations

from prettyqt import core
from prettyqt.qt import QtCore


QtCore.QStringListModel.__bases__ = (core.AbstractListModel,)


class StringListModel(QtCore.QStringListModel):
    def serialize_fields(self):
        return dict(
            string_list=self.stringList(),
        )



from __future__ import annotations

from prettyqt import core
from prettyqt.qt import QtCore


QtCore.QIdentityProxyModel.__bases__ = (core.AbstractProxyModel,)


class IdentityProxyModel(QtCore.QIdentityProxyModel):
    pass



from __future__ import annotations

import pathlib
from typing import TYPE_CHECKING, Any, Dict, List, Union


if TYPE_CHECKING:
    from prettyqt.qt import QtCore, QtGui, QtWidgets

    JSONType = Union[str, int, float, bool, None, Dict[str, Any], List[Any]]

    IconType = Union[QtGui.QIcon, str, pathlib.Path, None]

    ColorType = Union[str, int, QtCore.Qt.GlobalColor, QtGui.QColor, tuple, None]
    ColorAndBrushType = Union[ColorType, QtGui.QBrush]

    VariantType = Union[
        QtCore.QPersistentModelIndex,
        QtCore.QModelIndex,
        QtCore.QJsonDocument,
        # QtCore.QJsonArray,
        # QtCore.QJsonObject,
        QtCore.QJsonValue,
        QtCore.QUrl,
        QtCore.QUuid,
        QtCore.QEasingCurve,
        QtCore.QRegularExpression,
        QtCore.QLocale,
        QtCore.QRectF,
        QtCore.QRect,
        QtCore.QLineF,
        QtCore.QLine,
        QtCore.QPointF,
        QtCore.QPoint,
        QtCore.QSizeF,
        QtCore.QSize,
        QtCore.QDateTime,
        QtCore.QTime,
        QtCore.QDate,
        QtCore.QByteArray,
        QtGui.QColor,
        QtGui.QImage,
        QtGui.QBrush,
        QtGui.QBitmap,
        QtGui.QCursor,
        QtGui.QFont,
        QtGui.QKeySequence,
        QtGui.QIcon,
        QtGui.QTransform,
        QtGui.QPalette,
        QtGui.QMatrix4x4,
        QtGui.QPen,
        QtGui.QPixmap,
        QtGui.QPolygon,
        QtGui.QPolygonF,
        QtGui.QRegion,
        QtWidgets.QSizePolicy,
        QtGui.QTextFormat,
        QtGui.QTextLength,
        QtGui.QVector2D,
        QtGui.QVector3D,
        QtGui.QVector4D,
        bool,
        str,
        int,
        float,
        bytes,
    ]

    Variant = Union[VariantType, List[VariantType], Dict[str, VariantType]]



from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


QtWidgets.QStyleOptionTitleBar.__bases__ = (widgets.StyleOptionComplex,)


class StyleOptionTitleBar(QtWidgets.QStyleOptionTitleBar):
    pass



from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


QtWidgets.QStyleOptionGraphicsItem.__bases__ = (widgets.StyleOption,)


class StyleOptionGraphicsItem(QtWidgets.QStyleOptionGraphicsItem):
    pass



from __future__ import annotations

from prettyqt import core, widgets


class HoverIconEventFilter(core.Object):
    def __init__(self, normal, hover, parent=None):
        super().__init__(parent)
        self.normal = normal
        self.hover = hover

    def eventFilter(self, obj, event: core.Event):
        if event.type() == core.Event.Enter:
            obj.set_icon(self.hover)
        elif event.type() == core.Event.Leave:
            obj.set_icon(self.normal)
        return super().eventFilter(obj, event)


if __name__ == "__main__":
    app = widgets.app()
    widget = widgets.PushButton()
    test = HoverIconEventFilter("mdi.timer", "mdi.folder")
    widget.installEventFilter(test)
    widget.show()
    app.main_loop()



from __future__ import annotations

from typing import Optional, Union

from prettyqt import core, location
from prettyqt.qt import QtCore, QtLocation


class PlaceSupplier(QtLocation.QPlaceSupplier):
    def __bool__(self):
        return not self.isEmpty()

    def get_icon(self) -> Optional[location.PlaceIcon]:
        icon = self.icon()
        if icon.isEmpty():
            return None
        return location.PlaceIcon(icon)

    def set_url(self, url: Union[str, QtCore.QUrl]):
        url = core.Url(url)
        self.setUrl(url)

    def get_url(self) -> core.Url:
        return core.Url(self.url())


if __name__ == "__main__":
    supplier = PlaceSupplier()
    print(bool(supplier))



"""Constants module."""
from typing import Literal

from bidict import bidict
from prettyqt.qt import QtCore

from prettyqt.utils import mappers


DISPLAY_ROLE = QtCore.Qt.DisplayRole
USER_ROLE = QtCore.Qt.UserRole
SORT_ROLE = QtCore.Qt.UserRole + 1
NAME_ROLE = QtCore.Qt.UserRole + 2
EDIT_ROLE = QtCore.Qt.EditRole
BACKGROUND_ROLE = QtCore.Qt.BackgroundRole
FOREGROUND_ROLE = QtCore.Qt.ForegroundRole
TOOLTIP_ROLE = QtCore.Qt.ToolTipRole
STATUSTIP_ROLE = QtCore.Qt.StatusTipRole
DECORATION_ROLE = QtCore.Qt.DecorationRole
CHECKSTATE_ROLE = QtCore.Qt.CheckStateRole
ALIGNMENT_ROLE = QtCore.Qt.TextAlignmentRole
FONT_ROLE = QtCore.Qt.FontRole

ALIGN_LEFT = QtCore.Qt.AlignLeft
ALIGN_RIGHT = QtCore.Qt.AlignRight
ALIGN_H_CENTER = QtCore.Qt.AlignHCenter
ALIGN_JUSTIFY = QtCore.Qt.AlignJustify

ALIGN_TOP = QtCore.Qt.AlignTop
ALIGN_BOTTOM = QtCore.Qt.AlignBottom
ALIGN_V_CENTER = QtCore.Qt.AlignVCenter
ALIGN_BASELINE = QtCore.Qt.AlignBaseline

ALIGN_CENTER = QtCore.Qt.AlignCenter
ALIGN_CENTER_LEFT = QtCore.Qt.AlignVCenter | QtCore.Qt.AlignLeft
ALIGN_CENTER_RIGHT = QtCore.Qt.AlignVCenter | QtCore.Qt.AlignRight
ALIGN_TOP_LEFT = QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft
ALIGN_TOP_RIGHT = QtCore.Qt.AlignTop | QtCore.Qt.AlignRight
ALIGN_TOP_CENTER = QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter
ALIGN_BOTTOM_LEFT = QtCore.Qt.AlignBottom | QtCore.Qt.AlignLeft
ALIGN_BOTTOM_RIGHT = QtCore.Qt.AlignBottom | QtCore.Qt.AlignRight
ALIGN_BOTTOM_CENTER = QtCore.Qt.AlignBottom | QtCore.Qt.AlignHCenter

HORIZONTAL = QtCore.Qt.Horizontal
VERTICAL = QtCore.Qt.Vertical

ASCENDING = QtCore.Qt.AscendingOrder
DESCENDING = QtCore.Qt.DescendingOrder

DROP_ENABLED = QtCore.Qt.ItemIsDropEnabled
DRAG_ENABLED = QtCore.Qt.ItemIsDragEnabled
IS_ENABLED = QtCore.Qt.ItemIsEnabled
IS_SELECTABLE = QtCore.Qt.ItemIsSelectable
IS_EDITABLE = QtCore.Qt.ItemIsEditable
IS_CHECKABLE = QtCore.Qt.ItemIsUserCheckable
IS_AUTO_TRISTATE = QtCore.Qt.ItemIsAutoTristate
IS_USER_TRISTATE = QtCore.Qt.ItemIsUserTristate
NO_FLAGS = QtCore.Qt.NoItemFlags
NO_CHILDREN = QtCore.Qt.ItemNeverHasChildren

MOVE_ACTION = QtCore.Qt.MoveAction
COPY_ACTION = QtCore.Qt.CopyAction

TEXT_WORD_WRAP = QtCore.Qt.TextWordWrap

CTRL_MOD = QtCore.Qt.ControlModifier

KEY_F11 = QtCore.Qt.Key_F11
KEY_DELETE = QtCore.Qt.Key_Delete


ALIGNMENTS = mappers.FlagMap(
    QtCore.Qt.Alignment,
    # none=int(QtCore.Qt.Alignment(0)),
    left=ALIGN_LEFT,
    right=ALIGN_RIGHT,
    top=ALIGN_TOP,
    bottom=ALIGN_BOTTOM,
    top_left=ALIGN_TOP_LEFT,
    top_right=ALIGN_TOP_RIGHT,
    bottom_left=ALIGN_BOTTOM_LEFT,
    bottom_right=ALIGN_BOTTOM_RIGHT,
    center=ALIGN_CENTER,
)

AlignmentStr = Literal[
    "top",
    "left",
    "right",
    "bottom",
    "top_left",
    "top_right",
    "bottom_left",
    "bottom_right",
]

SIDES = mappers.FlagMap(
    QtCore.Qt.Alignment,
    left=QtCore.Qt.AlignLeft,
    right=QtCore.Qt.AlignRight,
    top=QtCore.Qt.AlignTop,
    bottom=QtCore.Qt.AlignBottom,
)

SideStr = Literal["left", "right", "top", "bottom"]

EDGES = mappers.FlagMap(
    QtCore.Qt.Edges,
    top=QtCore.Qt.TopEdge,
    left=QtCore.Qt.LeftEdge,
    right=QtCore.Qt.RightEdge,
    bottom=QtCore.Qt.BottomEdge,
    top_left=QtCore.Qt.TopEdge | QtCore.Qt.LeftEdge,
    top_right=QtCore.Qt.TopEdge | QtCore.Qt.RightEdge,
    bottom_left=QtCore.Qt.BottomEdge | QtCore.Qt.LeftEdge,
    bottom_right=QtCore.Qt.BottomEdge | QtCore.Qt.RightEdge,
)

EdgeStr = Literal[
    "top",
    "left",
    "right",
    "bottom",
    "top_left",
    "top_right",
    "bottom_left",
    "bottom_right",
]

H_ALIGNMENT = mappers.FlagMap(
    QtCore.Qt.Alignment,
    left=QtCore.Qt.AlignLeft,
    right=QtCore.Qt.AlignRight,
    center=QtCore.Qt.AlignHCenter,
    justify=QtCore.Qt.AlignJustify,
)

HorizontalAlignmentStr = Literal[
    "left",
    "right",
    "center",
    "justify",
]

V_ALIGNMENT = mappers.FlagMap(
    QtCore.Qt.Alignment,
    top=QtCore.Qt.AlignTop,
    bottom=QtCore.Qt.AlignBottom,
    center=QtCore.Qt.AlignVCenter,
    baseline=QtCore.Qt.AlignBaseline,
)

VerticalAlignmentStr = Literal[
    "top",
    "bottom",
    "center",
    "baseline",
]

OrientationStr = Literal["horizontal", "vertical"]
ORIENTATION: bidict[OrientationStr, QtCore.Qt.Orientation] = bidict(
    horizontal=QtCore.Qt.Horizontal, vertical=QtCore.Qt.Vertical
)

StateStr = Literal["unchecked", "partial", "checked"]
STATE: bidict[StateStr, QtCore.Qt.CheckState] = bidict(
    unchecked=QtCore.Qt.Unchecked,
    partial=QtCore.Qt.PartiallyChecked,
    checked=QtCore.Qt.Checked,
)

MATCH_FLAGS = dict(
    exact=QtCore.Qt.MatchExactly,
    fixed_string=QtCore.Qt.MatchFixedString,
    contains=QtCore.Qt.MatchContains,
    starts_with=QtCore.Qt.MatchStartsWith,
    ends_with=QtCore.Qt.MatchEndsWith,
    case_sensitive=QtCore.Qt.MatchCaseSensitive,
    regex=QtCore.Qt.MatchRegularExpression,
    wildcard=QtCore.Qt.MatchWildcard,
    wrap=QtCore.Qt.MatchWrap,
    recursive=QtCore.Qt.MatchRecursive,
)

MatchFlagStr = Literal[
    "exact",
    "fixed_string",
    "contains",
    "starts_with",
    "ends_with",
    "case_sensitive",
    "regex",
    "wildcard",
    "wrap",
    "recursive",
]

FilterModeStr = Literal["starts_with", "contains", "ends_with"]
FILTER_MODES = mappers.FlagMap(
    QtCore.Qt.MatchFlags,
    starts_with=QtCore.Qt.MatchStartsWith,
    contains=QtCore.Qt.MatchContains,
    ends_with=QtCore.Qt.MatchEndsWith,
)

DropActionStr = Literal["copy", "move", "link", "action_mask", "ignore", "target_move"]
DROP_ACTION: bidict[DropActionStr, QtCore.Qt.DropAction] = bidict(
    copy=QtCore.Qt.CopyAction,
    move=QtCore.Qt.MoveAction,
    link=QtCore.Qt.LinkAction,
    action_mask=QtCore.Qt.ActionMask,
    ignore=QtCore.Qt.IgnoreAction,
    target_move=QtCore.Qt.TargetMoveAction,
)

DockPositionStr = Literal["top", "bottom", "left", "right"]
DOCK_POSITION: bidict[DockPositionStr, QtCore.Qt.DockWidgetArea] = bidict(
    top=QtCore.Qt.TopDockWidgetArea,
    bottom=QtCore.Qt.BottomDockWidgetArea,
    left=QtCore.Qt.LeftDockWidgetArea,
    right=QtCore.Qt.RightDockWidgetArea,
)

DockPositionsStr = Literal["top", "bottom", "left", "right", "all"]
DOCK_POSITIONS: bidict[DockPositionsStr, QtCore.Qt.DockWidgetAreas] = bidict(
    top=QtCore.Qt.TopDockWidgetArea,
    bottom=QtCore.Qt.BottomDockWidgetArea,
    left=QtCore.Qt.LeftDockWidgetArea,
    right=QtCore.Qt.RightDockWidgetArea,
    all=QtCore.Qt.AllDockWidgetAreas,
)

ToolbarAreaStr = Literal["top", "bottom", "left", "right", "all", "none"]
TOOLBAR_AREA: bidict[ToolbarAreaStr, QtCore.Qt.ToolBarArea] = bidict(
    left=QtCore.Qt.LeftToolBarArea,
    right=QtCore.Qt.RightToolBarArea,
    top=QtCore.Qt.TopToolBarArea,
    bottom=QtCore.Qt.BottomToolBarArea,
    all=QtCore.Qt.AllToolBarAreas,
    none=QtCore.Qt.NoToolBarArea,
)

ToolButtonStyleStr = Literal["icon", "text", "text_beside_icon", "text_below_icon"]
TOOLBUTTON_STYLE: bidict[ToolButtonStyleStr, QtCore.Qt.ToolButtonStyle] = bidict(
    icon=QtCore.Qt.ToolButtonIconOnly,
    text=QtCore.Qt.ToolButtonTextOnly,
    text_beside_icon=QtCore.Qt.ToolButtonTextBesideIcon,
    text_below_icon=QtCore.Qt.ToolButtonTextUnderIcon,
)

WindowFrameSectionStr = Literal["none", "text", "text_beside_icon", "text_below_icon"]
WINDOW_FRAME_SECTION: bidict[
    WindowFrameSectionStr, QtCore.Qt.WindowFrameSection
] = bidict(
    none=QtCore.Qt.NoSection,
    left=QtCore.Qt.LeftSection,
    top_left=QtCore.Qt.TopLeftSection,
    top=QtCore.Qt.TopSection,
    top_right=QtCore.Qt.TopRightSection,
    right=QtCore.Qt.RightSection,
    bottom_right=QtCore.Qt.BottomRightSection,
    bottom=QtCore.Qt.BottomSection,
    bottom_left=QtCore.Qt.BottomLeftSection,
    title_bar=QtCore.Qt.TitleBarArea,
)

ArrowTypeStr = Literal["none", "up", "down", "left", "right"]
ARROW_TYPE: bidict[ArrowTypeStr, QtCore.Qt.ArrowType] = bidict(
    none=QtCore.Qt.NoArrow,
    up=QtCore.Qt.UpArrow,
    down=QtCore.Qt.DownArrow,
    left=QtCore.Qt.LeftArrow,
    right=QtCore.Qt.RightArrow,
)

EventPriorityStr = Literal["high", "normal", "low"]

# using int instead of QtCore.Qt.EventPriority here
EVENT_PRIORITY: bidict[EventPriorityStr, int] = bidict(
    high=1,  # HighEventPriority
    normal=0,  # NormalEventPriority
    low=-1,  # LowEventPriority
)

CursorShapeStr = Literal[
    "arrow",
    "uparrow",
    "cross",
    "wait",
    "caret",
    "size_vertical",
    "size_horizonal",
    "size_topright",
    "size_topleft",
    "size_all",
    "blank",
    "split_vertical",
    "split_horizontal",
    "pointing_hand",
    "forbidden",
    "open_hand",
    "closed_hand",
    "whats_this",
    "busy",
    "drag_move",
    "drag_copy",
    "drag_link",
    "bitmap",
]
CURSOR_SHAPE: bidict[CursorShapeStr, QtCore.Qt.CursorShape] = bidict(
    arrow=QtCore.Qt.ArrowCursor,
    uparrow=QtCore.Qt.UpArrowCursor,
    cross=QtCore.Qt.CrossCursor,
    wait=QtCore.Qt.WaitCursor,
    caret=QtCore.Qt.IBeamCursor,
    size_vertical=QtCore.Qt.SizeVerCursor,
    size_horizonal=QtCore.Qt.SizeHorCursor,
    size_topright=QtCore.Qt.SizeBDiagCursor,
    size_topleft=QtCore.Qt.SizeFDiagCursor,
    size_all=QtCore.Qt.SizeAllCursor,
    blank=QtCore.Qt.BlankCursor,
    split_vertical=QtCore.Qt.SplitVCursor,
    split_horizontal=QtCore.Qt.SplitHCursor,
    pointing_hand=QtCore.Qt.PointingHandCursor,
    forbidden=QtCore.Qt.ForbiddenCursor,
    open_hand=QtCore.Qt.OpenHandCursor,
    closed_hand=QtCore.Qt.ClosedHandCursor,
    whats_this=QtCore.Qt.WhatsThisCursor,
    busy=QtCore.Qt.BusyCursor,
    drag_move=QtCore.Qt.DragMoveCursor,
    drag_copy=QtCore.Qt.DragCopyCursor,
    drag_link=QtCore.Qt.DragLinkCursor,
    bitmap=QtCore.Qt.BitmapCursor,
)

LayoutDirectionStr = Literal["left_to_right", "right_to_left", "auto"]
LAYOUT_DIRECTION: bidict[LayoutDirectionStr, QtCore.Qt.LayoutDirection] = bidict(
    left_to_right=QtCore.Qt.LeftToRight,
    right_to_left=QtCore.Qt.RightToLeft,
    auto=QtCore.Qt.LayoutDirectionAuto,
)

ApplicationStateStr = Literal["suspended", "hidden", "inactive", "active"]
APPLICATION_STATES: bidict[ApplicationStateStr, QtCore.Qt.ApplicationState] = bidict(
    suspended=QtCore.Qt.ApplicationSuspended,
    hidden=QtCore.Qt.ApplicationHidden,
    inactive=QtCore.Qt.ApplicationInactive,
    active=QtCore.Qt.ApplicationActive,
)

HighDpiScaleFactorRoundingPolicyStr = Literal[
    "round", "ceil", "floor", "round_prefer_floor", "pass_through"
]
HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY: bidict[
    HighDpiScaleFactorRoundingPolicyStr, QtCore.Qt.HighDpiScaleFactorRoundingPolicy
] = bidict(
    round=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.Round,
    ceil=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.Ceil,
    floor=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.Floor,
    round_prefer_floor=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.RoundPreferFloor,
    pass_through=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.PassThrough,
)

UiEffectStr = Literal[
    "animate_menu",
    "fade_menu",
    "animate_combo",
    "animate_tooltip",
    "fade_tooltip",
    "animate_toolbox",
]
UI_EFFECTS: bidict[UiEffectStr, QtCore.Qt.UIEffect] = bidict(
    animate_menu=QtCore.Qt.UI_AnimateMenu,
    fade_menu=QtCore.Qt.UI_FadeMenu,
    animate_combo=QtCore.Qt.UI_AnimateCombo,
    animate_tooltip=QtCore.Qt.UI_AnimateTooltip,
    fade_tooltip=QtCore.Qt.UI_FadeTooltip,
    animate_toolbox=QtCore.Qt.UI_AnimateToolBox,
)

NavigationModeStr = Literal[
    "none",
    "keypad_tab_order",
    "keypad_directional",
    "cursor_auto",
    "cursor_force_visible",
]
NAVIGATION_MODES: bidict[NavigationModeStr, QtCore.Qt.NavigationMode] = bidict(
    none=QtCore.Qt.NavigationModeNone,
    keypad_tab_order=QtCore.Qt.NavigationModeKeypadTabOrder,
    keypad_directional=QtCore.Qt.NavigationModeKeypadDirectional,
    cursor_auto=QtCore.Qt.NavigationModeCursorAuto,
    cursor_force_visible=QtCore.Qt.NavigationModeCursorForceVisible,
)

ItemSelectionModeStr = Literal[
    "contains_shape",
    "intersects_shape",
    "contains_bounding_rect",
    "intersects_bounding_rect",
]
ITEM_SELECTION_MODE: bidict[ItemSelectionModeStr, QtCore.Qt.ItemSelectionMode] = bidict(
    contains_shape=QtCore.Qt.ContainsItemShape,
    intersects_shape=QtCore.Qt.IntersectsItemShape,
    contains_bounding_rect=QtCore.Qt.ContainsItemBoundingRect,
    intersects_bounding_rect=QtCore.Qt.IntersectsItemBoundingRect,
)

FocusReasonStr = Literal[
    "mouse", "tab", "backtab", "active_window", "popup", "shortcut", "menu_bar", "other"
]
FOCUS_REASONS: bidict[FocusReasonStr, QtCore.Qt.FocusReason] = bidict(
    mouse=QtCore.Qt.MouseFocusReason,
    tab=QtCore.Qt.TabFocusReason,
    backtab=QtCore.Qt.BacktabFocusReason,
    active_window=QtCore.Qt.ActiveWindowFocusReason,
    popup=QtCore.Qt.PopupFocusReason,
    shortcut=QtCore.Qt.ShortcutFocusReason,
    menu_bar=QtCore.Qt.MenuBarFocusReason,
    other=QtCore.Qt.OtherFocusReason,
)

ElideModeStr = Literal["left", "right", "middle", "none"]
ELIDE_MODE: bidict[ElideModeStr, QtCore.Qt.TextElideMode] = bidict(
    left=QtCore.Qt.ElideLeft,
    right=QtCore.Qt.ElideRight,
    middle=QtCore.Qt.ElideMiddle,
    none=QtCore.Qt.ElideNone,
)

PenStyleStr = Literal[
    "none", "solid", "dash", "dot", "dash_dot", "dash_dot_dot", "custom_dash"
]
PEN_STYLE: bidict[PenStyleStr, QtCore.Qt.PenStyle] = bidict(
    none=QtCore.Qt.NoPen,
    solid=QtCore.Qt.SolidLine,
    dash=QtCore.Qt.DashLine,
    dot=QtCore.Qt.DotLine,
    dash_dot=QtCore.Qt.DashDotLine,
    dash_dot_dot=QtCore.Qt.DashDotDotLine,
    custom_dash=QtCore.Qt.CustomDashLine,
)

CapStyleStr = Literal["flat", "square", "round"]
CAP_STYLE: bidict[CapStyleStr, QtCore.Qt.PenCapStyle] = bidict(
    flat=QtCore.Qt.FlatCap, square=QtCore.Qt.SquareCap, round=QtCore.Qt.RoundCap
)

JoinStyleStr = Literal["miter", "bevel", "round" "svg_miter"]
JOIN_STYLE: bidict[JoinStyleStr, QtCore.Qt.PenJoinStyle] = bidict(
    miter=QtCore.Qt.MiterJoin,
    bevel=QtCore.Qt.BevelJoin,
    round=QtCore.Qt.RoundJoin,
    svg_miter=QtCore.Qt.SvgMiterJoin,
)

PatternStr = Literal[
    "none",
    "solid",
    "dense_1",
    "dense_2",
    "dense_3",
    "dense_4",
    "dense_5",
    "dense_6",
    "dense_7",
    "horizontal",
    "vertical",
    "cross",
    "backward_diagonal",
    "forward_diagonal",
    "crossing_diagonal",
    "linear_gradient",
    "conical_gradient",
    "radial_gradient",
    "texture",
]

PATTERN: bidict[PatternStr, QtCore.Qt.BrushStyle] = bidict(
    none=QtCore.Qt.NoBrush,
    solid=QtCore.Qt.SolidPattern,
    dense_1=QtCore.Qt.Dense1Pattern,
    dense_2=QtCore.Qt.Dense2Pattern,
    dense_3=QtCore.Qt.Dense3Pattern,
    dense_4=QtCore.Qt.Dense4Pattern,
    dense_5=QtCore.Qt.Dense5Pattern,
    dense_6=QtCore.Qt.Dense6Pattern,
    dense_7=QtCore.Qt.Dense7Pattern,
    horizontal=QtCore.Qt.HorPattern,
    vertical=QtCore.Qt.VerPattern,
    cross=QtCore.Qt.CrossPattern,
    backward_diagonal=QtCore.Qt.BDiagPattern,
    forward_diagonal=QtCore.Qt.FDiagPattern,
    crossing_diagonal=QtCore.Qt.DiagCrossPattern,
    linear_gradient=QtCore.Qt.LinearGradientPattern,
    conical_gradient=QtCore.Qt.ConicalGradientPattern,
    radial_gradient=QtCore.Qt.RadialGradientPattern,
    texture=QtCore.Qt.TexturePattern,
)

ClipOperationStr = Literal["none", "replace", "intersect"]
CLIP_OPERATION: bidict[ClipOperationStr, QtCore.Qt.ClipOperation] = bidict(
    none=QtCore.Qt.NoClip,
    replace=QtCore.Qt.ReplaceClip,
    intersect=QtCore.Qt.IntersectClip,
)

ContextStr = Literal["widget", "widget_with_children", "window", "application"]
CONTEXT: bidict[ContextStr, QtCore.Qt.ShortcutContext] = bidict(
    widget=QtCore.Qt.WidgetShortcut,
    widget_with_children=QtCore.Qt.WidgetWithChildrenShortcut,
    window=QtCore.Qt.WindowShortcut,
    application=QtCore.Qt.ApplicationShortcut,
)

TileRuleStr = Literal["stretch", "repeat", "round"]
TILE_RULE: bidict[TileRuleStr, QtCore.Qt.TileRule] = bidict(
    stretch=QtCore.Qt.StretchTile,
    repeat=QtCore.Qt.RepeatTile,
    round=QtCore.Qt.RoundTile,
)

TransformationModeStr = Literal["fast", "smooth"]
TRANSFORMATION_MODE: bidict[TransformationModeStr, QtCore.Qt.TransformationMode] = bidict(
    fast=QtCore.Qt.FastTransformation,
    smooth=QtCore.Qt.SmoothTransformation,
)

GestureTypeStr = Literal["tap", "tap_and_hold", "pan", "pinch", "swipe", "custom"]
GESTURE_TYPE: bidict[GestureTypeStr, QtCore.Qt.GestureType] = bidict(
    tap=QtCore.Qt.TapGesture,
    tap_and_hold=QtCore.Qt.TapAndHoldGesture,
    pan=QtCore.Qt.PanGesture,
    pinch=QtCore.Qt.PinchGesture,
    swipe=QtCore.Qt.SwipeGesture,
    custom=QtCore.Qt.CustomGesture,
)

GestureStateStr = Literal["none", "started", "updated", "finished", "canceled"]
GESTURE_STATE: bidict[GestureStateStr, QtCore.Qt.GestureState] = bidict(
    none=QtCore.Qt.GestureState(0),  # type: ignore # QtCore.Qt.NoGesture,
    started=QtCore.Qt.GestureStarted,
    updated=QtCore.Qt.GestureUpdated,
    finished=QtCore.Qt.GestureFinished,
    canceled=QtCore.Qt.GestureCanceled,
)

ScrollBarPolicyStr = Literal["always_on", "always_off", "as_needed"]
SCROLLBAR_POLICY: bidict[ScrollBarPolicyStr, QtCore.Qt.ScrollBarPolicy] = bidict(
    always_on=QtCore.Qt.ScrollBarAlwaysOn,
    always_off=QtCore.Qt.ScrollBarAlwaysOff,
    as_needed=QtCore.Qt.ScrollBarAsNeeded,
)

ContextPolicyStr = Literal["none", "prevent", "default", "actions", "custom"]
CONTEXT_POLICY: bidict[ContextPolicyStr, QtCore.Qt.ContextMenuPolicy] = bidict(
    none=QtCore.Qt.NoContextMenu,
    prevent=QtCore.Qt.PreventContextMenu,
    default=QtCore.Qt.DefaultContextMenu,
    actions=QtCore.Qt.ActionsContextMenu,
    custom=QtCore.Qt.CustomContextMenu,
    # showhide_menu="showhide_menu",
)

ModalityStr = Literal["window", "application", "none"]
MODALITY: bidict[ModalityStr, QtCore.Qt.WindowModality] = bidict(
    window=QtCore.Qt.WindowModal,
    application=QtCore.Qt.ApplicationModal,
    none=QtCore.Qt.NonModal,
)

FocusPolicyStr = Literal["tab", "click", "strong", "wheel", "none"]
FOCUS_POLICY: bidict[FocusPolicyStr, QtCore.Qt.FocusPolicy] = bidict(
    tab=QtCore.Qt.TabFocus,
    click=QtCore.Qt.ClickFocus,
    strong=QtCore.Qt.StrongFocus,
    wheel=QtCore.Qt.WheelFocus,
    none=QtCore.Qt.NoFocus,
)

WindowFlagStr = Literal[
    "frameless", "popup", "stay_on_top", "tool", "window_title", "customize_window"
]

WINDOW_FLAGS: bidict[WindowFlagStr, QtCore.Qt.WindowType] = bidict(
    frameless=QtCore.Qt.FramelessWindowHint,
    popup=QtCore.Qt.Popup,
    stay_on_top=QtCore.Qt.WindowStaysOnTopHint,
    tool=QtCore.Qt.Tool,
    window_title=QtCore.Qt.WindowTitleHint,
    customize_window=QtCore.Qt.CustomizeWindowHint,
)

WindowStateStr = Literal["none", "minimized", "maximized", "fullscreen", "active"]
WINDOW_STATES = mappers.FlagMap(
    QtCore.Qt.WindowStates,
    none=QtCore.Qt.WindowNoState,
    minimized=QtCore.Qt.WindowMinimized,
    maximized=QtCore.Qt.WindowMaximized,
    fullscreen=QtCore.Qt.WindowFullScreen,
    active=QtCore.Qt.WindowActive,
)

FillRuleStr = Literal["odd_even", "winding"]
FILL_RULE: bidict[FillRuleStr, QtCore.Qt.FillRule] = bidict(
    odd_even=QtCore.Qt.OddEvenFill, winding=QtCore.Qt.WindingFill
)

TimerTypeStr = Literal["precise", "coarse", "very_coarse"]
TIMER_TYPE: bidict[TimerTypeStr, QtCore.Qt.TimerType] = bidict(
    precise=QtCore.Qt.PreciseTimer,
    coarse=QtCore.Qt.CoarseTimer,
    very_coarse=QtCore.Qt.VeryCoarseTimer,
)

CursorMoveStyleStr = Literal["logical", "visual"]
CURSOR_MOVE_STYLE: bidict[CursorMoveStyleStr, QtCore.Qt.CursorMoveStyle] = bidict(
    logical=QtCore.Qt.LogicalMoveStyle, visual=QtCore.Qt.VisualMoveStyle
)

CornerStr = Literal["top_left", "top_right", "bottom_left", "bottom_right"]
CORNER: bidict[CornerStr, QtCore.Qt.Corner] = bidict(
    top_left=QtCore.Qt.TopLeftCorner,
    top_right=QtCore.Qt.TopRightCorner,
    bottom_left=QtCore.Qt.BottomLeftCorner,
    bottom_right=QtCore.Qt.BottomRightCorner,
)

ScreenOrientationStr = Literal[
    "primary", "landscape", "portrait", "inverted_landscape", "inverted_portrait"
]
SCREEN_ORIENTATION: bidict[ScreenOrientationStr, QtCore.Qt.ScreenOrientation] = bidict(
    primary=QtCore.Qt.PrimaryOrientation,
    landscape=QtCore.Qt.LandscapeOrientation,
    portrait=QtCore.Qt.PortraitOrientation,
    inverted_landscape=QtCore.Qt.InvertedLandscapeOrientation,
    inverted_portrait=QtCore.Qt.InvertedPortraitOrientation,
)

AspectRatioModeStr = Literal["ignore", "keep", "keep_by_expanding"]
ASPECT_RATIO_MODE: bidict[AspectRatioModeStr, QtCore.Qt.AspectRatioMode] = bidict(
    ignore=QtCore.Qt.IgnoreAspectRatio,
    keep=QtCore.Qt.KeepAspectRatio,
    keep_by_expanding=QtCore.Qt.KeepAspectRatioByExpanding,
)

DateFormatStr = Literal["text", "iso", "iso_with_ms", "rfc_2822"]
DATE_FORMAT: bidict[DateFormatStr, QtCore.Qt.DateFormat] = bidict(
    text=QtCore.Qt.TextDate,
    iso=QtCore.Qt.ISODate,
    iso_with_ms=QtCore.Qt.ISODateWithMs,
    rfc_2822=QtCore.Qt.RFC2822Date,
)

TimeSpecStr = Literal["local_time", "utc", "offset_from_utc", "timezone"]
TIME_SPEC: bidict[TimeSpecStr, QtCore.Qt.TimeSpec] = bidict(
    local_time=QtCore.Qt.LocalTime,
    utc=QtCore.Qt.UTC,
    offset_from_utc=QtCore.Qt.OffsetFromUTC,
    timezone=QtCore.Qt.TimeZone,
)

AxisStr = Literal["x", "y", "z"]
AXIS: bidict[AxisStr, QtCore.Qt.Axis] = bidict(
    x=QtCore.Qt.XAxis, y=QtCore.Qt.YAxis, z=QtCore.Qt.ZAxis
)

WidgetAttributeStr = Literal[
    "accept_drops",
    "always_show_tooltips",
    "custom_whats_this",
    "delete_on_close",
    "disabled",
    "dont_show_on_screen",
    "force_disabled",
    "force_updates_disabled",
    "hover",
    "input_method_enabled",
    "keyboard_focus_change",
    "key_compression",
    "layout_on_entire_rect",
    "layout_uses_widget_rect",
    "mac_opaque_size_grip",
    "mac_show_focus_rect",
    "mac_normal_size",
    "mac_small_size",
    "mac_mini_size",
    # "mac_variable_size",
    "mapped",
    "mouse_no_mask",
    "mouse_tracking",
    "moved",
    "no_child_events_for_parent",
    "no_child_events_from_children",
    "no_mouse_replay",
    "no_mouse_propagation",
    "transparent_for_mouse_events",
    "no_system_background",
    "opaque_paint_event",
    "outside_ws_range",
    "paint_on_screen",
    "paint_unclipped",
    "pending_move_event",
    "pending_resize_egent",
    "quit_on_close",
    "resized",
    "right_to_left",
    "set_cursor",
    "set_font",
    "set_palette",
    "set_style",
    "show_modal",
    "static_contents",
    "style_sheet",
    "style_sheet_target",
    "tablet_tracking",
    "translucent_background",
    "under_mouse",
    "updates_disabled",
    "window_modified",
    "window_propagation",
    "mac_always_show_tool_window",
    "set_locale",
    "styled_background",
    "show_without_activating",
    "native_window",
    "dont_create_native_ancestors",
    "x11_net_wm_window_type_desktop",
    "x11_net_wm_window_type_dock",
    "x11_net_wm_window_type_toolbar",
    "x11_net_wm_window_type_menu",
    "x11_net_wm_window_type_utility",
    "x11_net_wm_window_type_splash",
    "x11_net_wm_window_type_dialog",
    "x11_net_wm_window_type_dropdown_menu",
    "x11_net_wm_window_type_popup_menu",
    "x11_net_wm_window_type_tooltip",
    "x11_net_wm_window_type_notification",
    "x11_net_wm_window_type_combo",
    "x11_net_wm_window_type_dnd",
    "accept_touch_events",
    "touch_pad_single_touch_events",
    "x11_do_not_accept_focus",
    "always_stack_on_top",
    "contents_margins_respects_safe_area",
]
WIDGET_ATTRIBUTE: bidict[WidgetAttributeStr, QtCore.Qt.WidgetAttribute] = bidict(
    accept_drops=QtCore.Qt.WA_AcceptDrops,
    always_show_tooltips=QtCore.Qt.WA_AlwaysShowToolTips,
    custom_whats_this=QtCore.Qt.WA_CustomWhatsThis,
    delete_on_close=QtCore.Qt.WA_DeleteOnClose,
    disabled=QtCore.Qt.WA_Disabled,
    dont_show_on_screen=QtCore.Qt.WA_DontShowOnScreen,
    force_disabled=QtCore.Qt.WA_ForceDisabled,
    force_updates_disabled=QtCore.Qt.WA_ForceUpdatesDisabled,
    hover=QtCore.Qt.WA_Hover,
    input_method_enabled=QtCore.Qt.WA_InputMethodEnabled,
    keyboard_focus_change=QtCore.Qt.WA_KeyboardFocusChange,
    key_compression=QtCore.Qt.WA_KeyCompression,
    layout_on_entire_rect=QtCore.Qt.WA_LayoutOnEntireRect,
    layout_uses_widget_rect=QtCore.Qt.WA_LayoutUsesWidgetRect,
    mac_opaque_size_grip=QtCore.Qt.WA_MacOpaqueSizeGrip,
    mac_show_focus_rect=QtCore.Qt.WA_MacShowFocusRect,
    mac_normal_size=QtCore.Qt.WA_MacNormalSize,
    mac_small_size=QtCore.Qt.WA_MacSmallSize,
    mac_mini_size=QtCore.Qt.WA_MacMiniSize,
    # mac_variable_size=QtCore.Qt.WA_MacVariableSize,
    mapped=QtCore.Qt.WA_Mapped,
    mouse_no_mask=QtCore.Qt.WA_MouseNoMask,
    mouse_tracking=QtCore.Qt.WA_MouseTracking,
    moved=QtCore.Qt.WA_Moved,
    no_child_events_for_parent=QtCore.Qt.WA_NoChildEventsForParent,
    no_child_events_from_children=QtCore.Qt.WA_NoChildEventsFromChildren,
    no_mouse_replay=QtCore.Qt.WA_NoMouseReplay,
    no_mouse_propagation=QtCore.Qt.WA_NoMousePropagation,
    transparent_for_mouse_events=QtCore.Qt.WA_TransparentForMouseEvents,
    no_system_background=QtCore.Qt.WA_NoSystemBackground,
    opaque_paint_event=QtCore.Qt.WA_OpaquePaintEvent,
    outside_ws_range=QtCore.Qt.WA_OutsideWSRange,
    paint_on_screen=QtCore.Qt.WA_PaintOnScreen,
    paint_unclipped=QtCore.Qt.WA_PaintUnclipped,
    pending_move_event=QtCore.Qt.WA_PendingMoveEvent,
    pending_resize_egent=QtCore.Qt.WA_PendingResizeEvent,
    quit_on_close=QtCore.Qt.WA_QuitOnClose,
    resized=QtCore.Qt.WA_Resized,
    right_to_left=QtCore.Qt.WA_RightToLeft,
    set_cursor=QtCore.Qt.WA_SetCursor,
    set_font=QtCore.Qt.WA_SetFont,
    set_palette=QtCore.Qt.WA_SetPalette,
    set_style=QtCore.Qt.WA_SetStyle,
    # show_modal=QtCore.Qt.WA_ShowModal,
    static_contents=QtCore.Qt.WA_StaticContents,
    style_sheet=QtCore.Qt.WA_StyleSheet,
    style_sheet_target=QtCore.Qt.WA_StyleSheetTarget,
    tablet_tracking=QtCore.Qt.WA_TabletTracking,
    translucent_background=QtCore.Qt.WA_TranslucentBackground,
    under_mouse=QtCore.Qt.WA_UnderMouse,
    updates_disabled=QtCore.Qt.WA_UpdatesDisabled,
    window_modified=QtCore.Qt.WA_WindowModified,
    window_propagation=QtCore.Qt.WA_WindowPropagation,
    mac_always_show_tool_window=QtCore.Qt.WA_MacAlwaysShowToolWindow,
    set_locale=QtCore.Qt.WA_SetLocale,
    styled_background=QtCore.Qt.WA_StyledBackground,
    show_without_activating=QtCore.Qt.WA_ShowWithoutActivating,
    native_window=QtCore.Qt.WA_NativeWindow,
    dont_create_native_ancestors=QtCore.Qt.WA_DontCreateNativeAncestors,
    x11_net_wm_window_type_desktop=QtCore.Qt.WA_X11NetWmWindowTypeDesktop,
    x11_net_wm_window_type_dock=QtCore.Qt.WA_X11NetWmWindowTypeDock,
    x11_net_wm_window_type_toolbar=QtCore.Qt.WA_X11NetWmWindowTypeToolBar,
    x11_net_wm_window_type_menu=QtCore.Qt.WA_X11NetWmWindowTypeMenu,
    x11_net_wm_window_type_utility=QtCore.Qt.WA_X11NetWmWindowTypeUtility,
    x11_net_wm_window_type_splash=QtCore.Qt.WA_X11NetWmWindowTypeSplash,
    x11_net_wm_window_type_dialog=QtCore.Qt.WA_X11NetWmWindowTypeDialog,
    x11_net_wm_window_type_dropdown_menu=QtCore.Qt.WA_X11NetWmWindowTypeDropDownMenu,
    x11_net_wm_window_type_popup_menu=QtCore.Qt.WA_X11NetWmWindowTypePopupMenu,
    x11_net_wm_window_type_tooltip=QtCore.Qt.WA_X11NetWmWindowTypeToolTip,
    x11_net_wm_window_type_notification=QtCore.Qt.WA_X11NetWmWindowTypeNotification,
    x11_net_wm_window_type_combo=QtCore.Qt.WA_X11NetWmWindowTypeCombo,
    x11_net_wm_window_type_dnd=QtCore.Qt.WA_X11NetWmWindowTypeDND,
    accept_touch_events=QtCore.Qt.WA_AcceptTouchEvents,
    touch_pad_single_touch_events=QtCore.Qt.WA_TouchPadAcceptSingleTouchEvents,
    x11_do_not_accept_focus=QtCore.Qt.WA_X11DoNotAcceptFocus,
    always_stack_on_top=QtCore.Qt.WA_AlwaysStackOnTop,
    contents_margins_respects_safe_area=QtCore.Qt.WA_ContentsMarginsRespectsSafeArea,
)



from __future__ import annotations

from prettyqt import charts
from prettyqt.qt.QtCharts import QtCharts


QtCharts.QHorizontalStackedBarSeries.__bases__ = (charts.AbstractBarSeries,)


class HorizontalStackedBarSeries(QtCharts.QHorizontalStackedBarSeries):
    pass



from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets
from prettyqt.utils import bidict


QtWidgets.QMdiSubWindow.__bases__ = (widgets.Widget,)


SUB_WINDOW_OPTION = bidict(
    rubber_band_resize=QtWidgets.QMdiSubWindow.RubberBandResize,
    rubber_band_move=QtWidgets.QMdiSubWindow.RubberBandMove,
)


class MdiSubWindow(QtWidgets.QMdiSubWindow):
    def serialize_fields(self):
        return dict(
            keyboard_single_step=self.keyboardSingleStep(),
            keyboard_page_step=self.keyboardPageStep(),
        )


if __name__ == "__main__":
    app = widgets.app()
    widget = MdiSubWindow()
    widget.show()
    app.main_loop()



from __future__ import annotations

from prettyqt import core
from prettyqt.qt import QtWidgets


QtWidgets.QKeyEventTransition.__bases__ = (core.EventTransition,)


class KeyEventTransition(QtWidgets.QKeyEventTransition):
    pass



from __future__ import annotations

import os
from typing import Union

from prettyqt import core
from prettyqt.qt import QtCore, QtGui


class DesktopServices(QtGui.QDesktopServices):
    @classmethod
    def open_url(cls, location: Union[str, os.PathLike, QtCore.QUrl]) -> bool:
        if not isinstance(location, QtCore.QUrl):
            location = core.Url.from_user_input(os.fspath(location))
        return cls.openUrl(location)



from __future__ import annotations

import datetime
from typing import Union

from prettyqt import core, widgets
from prettyqt.qt import QtCore, QtWidgets


QtWidgets.QDateEdit.__bases__ = (widgets.DateTimeEdit,)


class DateEdit(QtWidgets.QDateEdit):

    value_changed = core.Signal(datetime.datetime)

    def __setstate__(self, state):
        super().__setstate__(state)
        self.setDate(state["date"])
        self.set_range(*state["range"])

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def serialize_fields(self):
        return dict(
            date=self.get_date(),
            range=(self.min_date(), self.max_date()),
        )

    def set_value(self, value: Union[QtCore.QDate, datetime.date]):
        self.setDate(value)  # type: ignore

    def set_range(
        self,
        lower: Union[QtCore.QDate, datetime.date],
        upper: Union[QtCore.QDate, datetime.date],
    ):
        self.setToolTip(f"{lower} <= x <= {upper}")
        self.setDateRange(lower, upper)  # type: ignore

    def get_value(self) -> datetime.date:
        return self.get_date()


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    widget = DateEdit()
    widget.show()
    app.main_loop()



from __future__ import annotations

from typing import Optional

from prettyqt import constants, core
from prettyqt.qt import QtCore, QtMultimedia


class PlaylistModel(core.AbstractTableModel):

    HEADER = ["Name"]

    def __init__(self, parent: Optional[QtCore.QObject] = None):
        super().__init__(parent)
        self._playlist: Optional[QtMultimedia.QMediaPlaylist] = None

    def rowCount(self, parent=None):
        if self._playlist is None:
            return 0
        return len(self._playlist)

    def columnCount(self, parent=None):
        return len(self.HEADER)

    def data(self, index: core.ModelIndex, role=constants.DISPLAY_ROLE):
        if not index.isValid():
            return None
        if self._playlist is None:
            return None
        if role == constants.DISPLAY_ROLE:
            if index.column() == 0:
                location = self._playlist.media(index.row()).canonicalUrl()
                return core.FileInfo(location.path()).fileName()
        return None

    def get_playlist(self) -> Optional[QtMultimedia.QMediaPlaylist]:
        return self._playlist

    def set_playlist(self, playlist: Optional[QtMultimedia.QMediaPlaylist]):
        if self._playlist is not None:
            self._playlist.mediaAboutToBeInserted.disconnect(self.beginInsertItems)
            self._playlist.mediaInserted.disconnect(self.endInsertRows)
            self._playlist.mediaAboutToBeRemoved.disconnect(self.beginRemoveItems)
            self._playlist.mediaRemoved.disconnect(self.endRemoveRows)
            self._playlist.mediaChanged.disconnect(self.change_items)

        with self.reset_model():
            self._playlist = playlist

            if self._playlist is not None:
                self._playlist.mediaAboutToBeInserted.connect(
                    lambda x, y: self.beginInsertRows(core.ModelIndex(), x, y)
                )
                self._playlist.mediaInserted.connect(self.endInsertRows)
                self._playlist.mediaAboutToBeRemoved.connect(
                    lambda x, y: self.beginRemoveRows(core.ModelIndex(), x, y)
                )
                self._playlist.mediaRemoved.connect(self.endRemoveRows)
                self._playlist.mediaChanged.connect(self.change_items)

    def change_items(self, start: int, end: int):
        self.dataChanged.emit(self.index(start, 0), self.index(end, len(self.HEADER)))


if __name__ == "__main__":
    from prettyqt import multimedia, widgets

    URL = (
        "http://commondatastorage.googleapis.com/"
        "gtv-videos-bucket/sample/BigBuckBunny.mp4"
    )

    app = widgets.app()
    tableview = widgets.TableView()
    model = PlaylistModel()
    playlist = multimedia.MediaPlaylist()
    playlist.add_media(URL)
    model.set_playlist(playlist)
    tableview.set_model(model)
    print(model.rowCount(), model.columnCount())
    print(model._playlist)
    tableview.show()
    app.main_loop()



from __future__ import annotations

from typing import Literal, Optional, Union

from prettyqt import widgets
from prettyqt.qt import QtWidgets
from prettyqt.utils import bidict


SHAPE = bidict(line=QtWidgets.QRubberBand.Line, rectangle=QtWidgets.QRubberBand.Rectangle)

ShapeStr = Literal["line", "rectangle"]


QtWidgets.QRubberBand.__bases__ = (widgets.Widget,)


class RubberBand(QtWidgets.QRubberBand):
    def __init__(
        self,
        shape: Union[ShapeStr, QtWidgets.QRubberBand.Shape],
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        shape = SHAPE[shape] if isinstance(shape, str) else shape
        super().__init__(shape, parent)

    def get_shape(self) -> ShapeStr:
        return SHAPE.inverse[self.shape()]



from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


QtWidgets.QStyleOptionComplex.__bases__ = (widgets.StyleOption,)


class StyleOptionComplex(QtWidgets.QStyleOptionComplex):
    pass



"""Webenginewidgets module.

contains QtWebEngineCore-based classes
"""

import logging

logger = logging.getLogger(__name__)

try:
    from .webchannel import WebChannel

    __all__ = ["WebChannel"]
except ModuleNotFoundError:
    logger.warning("Tried to import WebEngine module but not installed.")



from __future__ import annotations

from typing import Literal, Optional

from PyQt5 import Qsci, QtGui  # type: ignore

from prettyqt import gui, widgets
from prettyqt.utils import InvalidParamError, bidict, colors, types


ARROW_MARKER_NUM = 8

MATCH_TYPE = bidict(
    none=Qsci.QsciScintilla.NoBraceMatch,
    strict=Qsci.QsciScintilla.StrictBraceMatch,
    sloppy=Qsci.QsciScintilla.SloppyBraceMatch,
)

MatchTypeStr = Literal["none", "strict", "sloppy"]

LEXER = bidict(
    bash=Qsci.QsciLexerBash,
    batch=Qsci.QsciLexerBatch,
    cmake=Qsci.QsciLexerCMake,
    cpp=Qsci.QsciLexerCPP,
    css=Qsci.QsciLexerCSS,
    csharp=Qsci.QsciLexerCSharp,
    coffeescript=Qsci.QsciLexerCoffeeScript,
    d=Qsci.QsciLexerD,
    diff=Qsci.QsciLexerDiff,
    fortran=Qsci.QsciLexerFortran,
    fortran77=Qsci.QsciLexerFortran77,
    html=Qsci.QsciLexerHTML,
    java=Qsci.QsciLexerJava,
    javascript=Qsci.QsciLexerJavaScript,
    lua=Qsci.QsciLexerLua,
    makefile=Qsci.QsciLexerMakefile,
    matlab=Qsci.QsciLexerMatlab,
    octave=Qsci.QsciLexerOctave,
    po=Qsci.QsciLexerPO,
    pov=Qsci.QsciLexerPOV,
    pascal=Qsci.QsciLexerPascal,
    perl=Qsci.QsciLexerPerl,
    postscript=Qsci.QsciLexerPostScript,
    properties=Qsci.QsciLexerProperties,
    ruby=Qsci.QsciLexerRuby,
    spice=Qsci.QsciLexerSpice,
    tcl=Qsci.QsciLexerTCL,
    tex=Qsci.QsciLexerTeX,
    vhdl=Qsci.QsciLexerVHDL,
    verilog=Qsci.QsciLexerVerilog,
    xml=Qsci.QsciLexerXML,
    python=Qsci.QsciLexerPython,
    yaml=Qsci.QsciLexerYAML,
    json=Qsci.QsciLexerJSON,
    sql=Qsci.QsciLexerSQL,
    markdown=Qsci.QsciLexerMarkdown,
    idl=Qsci.QsciLexerIDL,
)

LexerStr = Literal[
    "bash",
    "batch",
    "cmake",
    "cpp",
    "css",
    "csharp",
    "coffeescript",
    "d",
    "diff",
    "fortran",
    "fortran77",
    "html",
    "java",
    "javascript",
    "lua",
    "makefile",
    "matlab",
    "octave",
    "po",
    "pov",
    "pascal",
    "perl",
    "postscript",
    "properties",
    "ruby",
    "spice",
    "tcl",
    "tex",
    "vhdl",
    "verilog",
    "xml",
    "python",
    "yaml",
    "json",
    "sql",
    "markdown",
    "idl",
]

MARKER = bidict(
    circle=Qsci.QsciScintilla.Circle,
    rectangle=Qsci.QsciScintilla.Rectangle,
    right_triangle=Qsci.QsciScintilla.RightTriangle,
    small_rectangle=Qsci.QsciScintilla.SmallRectangle,
    right_arrow=Qsci.QsciScintilla.RightArrow,
    invisible=Qsci.QsciScintilla.Invisible,
    down_triangle=Qsci.QsciScintilla.DownTriangle,
    minus=Qsci.QsciScintilla.Minus,
    plus=Qsci.QsciScintilla.Plus,
    vertical_line=Qsci.QsciScintilla.VerticalLine,
    bottom_left_corner=Qsci.QsciScintilla.BottomLeftCorner,
    left_side_splitter=Qsci.QsciScintilla.LeftSideSplitter,
    boxed_plus=Qsci.QsciScintilla.BoxedPlus,
    boxed_plus_connected=Qsci.QsciScintilla.BoxedPlusConnected,
    boxed_minus=Qsci.QsciScintilla.BoxedMinus,
    boxed_minus_connected=Qsci.QsciScintilla.BoxedMinusConnected,
    rounded_bottom_left_corner=Qsci.QsciScintilla.RoundedBottomLeftCorner,
    left_side_rounded_splitter=Qsci.QsciScintilla.LeftSideRoundedSplitter,
    circled_plus=Qsci.QsciScintilla.CircledPlus,
    circled_plus_connected=Qsci.QsciScintilla.CircledPlusConnected,
    circled_minus=Qsci.QsciScintilla.CircledMinus,
    circled_minus_connected=Qsci.QsciScintilla.CircledMinusConnected,
    background=Qsci.QsciScintilla.Background,
    three_dots=Qsci.QsciScintilla.ThreeDots,
    three_right_arrows=Qsci.QsciScintilla.ThreeRightArrows,
    full_rectangle=Qsci.QsciScintilla.FullRectangle,
    left_rectangle=Qsci.QsciScintilla.LeftRectangle,
    underline=Qsci.QsciScintilla.Underline,
    bookmark=Qsci.QsciScintilla.Bookmark,
)

MarkerStr = Literal[
    "circle",
    "rectangle",
    "right_triangle",
    "small_rectangle",
    "right_arrow",
    "invisible",
    "down_triangle",
    "minus",
    "plus",
    "vertical_line",
    "bottom_left_corner",
    "left_side_splitter",
    "boxed_plus",
    "boxed_plus_connected",
    "boxed_minus",
    "boxed_minus_connected",
    "rounded_bottom_left_corner",
    "left_side_rounded_splitter",
    "circled_plus",
    "circled_plus_connected",
    "circled_minus",
    "circled_minus_connected",
    "background",
    "three_dots",
    "three_right_arrows",
    "full_rectangle",
    "left_rectangle",
    "underline",
    "bookmark",
]

WRAP_MODE = bidict(
    none=Qsci.QsciScintilla.WrapNone,
    word=Qsci.QsciScintilla.WrapWord,
    anywhere=Qsci.QsciScintilla.WrapCharacter,
    whitespace=Qsci.QsciScintilla.WrapWhitespace,
)

WrapModeStr = Literal["none", "word", "anywhere", "whitespace"]

Qsci.QsciScintilla.__bases__[0].__bases__ = (widgets.AbstractScrollArea,)


class SciScintilla(Qsci.QsciScintilla):
    supported_langs = LEXER.keys()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Set the default font
        font = gui.Font("Consolas")
        self.setFont(font)
        self.setMarginsFont(font)

        # Margin 0 is used for line numbers
        self.setMarginWidth(0, font.metrics.width("00000") + 6)
        self.setMarginLineNumbers(0, True)
        self.set_margins_background_color("lightgrey")

        # Clickable margin 1 for showing markers
        # self.setMarginSensitivity(1, True)
        # self.marginClicked.connect(self.on_margin_clicked)
        # self.define_marker("right_arrow", ARROW_MARKER_NUM)
        # self.set_marker_background_color("red", ARROW_MARKER_NUM)
        self.set_brace_matching("sloppy")
        self.highlight_current_line()

        # Don't want to see the horizontal scrollbar at all
        # Use raw message to Scintilla here (all messages are documented
        # here: http://www.scintilla.org/ScintillaDoc.html)
        # self.SendScintilla(Qsci.QsciScintilla.SCI_SETHSCROLLBAR, 0)
        self.SendScintilla(self.SCI_SETSCROLLWIDTHTRACKING, True)
        self.SendScintilla(self.SCI_SETSCROLLWIDTH, 5)
        self.language = None
        self._lexer = None

    def set_marker_background_color(self, color: types.ColorType, marker_num):
        self.setMarkerBackgroundColor(colors.get_color(color), marker_num)

    def set_margins_background_color(self, color: types.ColorType):
        self.setMarginsBackgroundColor(colors.get_color(color))

    def highlight_current_line(self, color: types.ColorType = "yellow"):
        self.setCaretLineVisible(color is not None)
        self.setCaretLineBackgroundColor(colors.get_color(color))

    def set_brace_matching(self, match_type: Optional[MatchTypeStr]):
        if match_type is None:
            match_type = "none"
        if match_type not in MATCH_TYPE:
            raise InvalidParamError(match_type, MATCH_TYPE)
        self.setBraceMatching(MATCH_TYPE[match_type])

    def define_marker(self, marker: MarkerStr, num: int):
        if marker not in MARKER:
            raise InvalidParamError(marker, MARKER)
        self.markerDefine(MARKER[marker], num)

    def set_text(self, text: str):
        self.setText(text)

    def set_syntaxhighlighter(self, language: LexerStr):
        self.language = language
        font = gui.Font.mono()
        self._lexer = LEXER[language]()
        self._lexer.setDefaultFont(font)
        self._lexer.setFont(font)
        self.setLexer(self._lexer)

    def scroll_to_bottom(self):
        self.ensureLineVisible(self.lines())

    def append_text(self, text: str, newline: bool = True):
        self.append(f"\n{text}" if newline else text)

    def set_font(self, font: QtGui.QFont):
        self.setFont(font)
        self.setMarginsFont(font)
        lexer = self._lexer
        if lexer is not None:
            lexer.setDefaultFont(font)
            lexer.setFont(font)
            self.setLexer(lexer)

    def set_read_only(self, value: bool = True):
        self.setReadOnly(value)

    def on_margin_clicked(self, nmargin, nline, modifiers):
        self.toggle_marker(nline, ARROW_MARKER_NUM)

    def toggle_marker(self, nline: int, marker_num: int):
        # Toggle marker for the line the margin was clicked on
        if self.markersAtLine(nline) != 0:
            self.markerDelete(nline, marker_num)
        else:
            self.markerAdd(nline, marker_num)

    def set_wrap_mode(self, mode: Optional[WrapModeStr]):
        if mode is None:
            mode = "none"
        if mode not in WRAP_MODE:
            raise InvalidParamError(mode, WRAP_MODE)
        self.setWrapMode(WRAP_MODE[mode])

    def get_value(self) -> str:
        return self.text()

    def set_value(self, value: str):
        self.setText(value)


if __name__ == "__main__":
    app = widgets.app()
    widget = SciScintilla()
    widget.set_syntaxhighlighter("python")
    widget.set_wrap_mode("anywhere")
    widget.show()
    app.main_loop()



from __future__ import annotations

import os
from typing import List, Literal, Optional, Tuple, Union

from prettyqt import core, gui
from prettyqt.qt import QtCore, QtGui
from prettyqt.utils import InvalidParamError, bidict


ICON_ENGINE_HOOK = bidict(
    # available_sizes=QtGui.QIconEngine.AvailableSizesHook,
    # icon_name=QtGui.QIconEngine.IconNameHook,
    is_null=QtGui.QIconEngine.IsNullHook,
    scaled_pixmap=QtGui.QIconEngine.ScaledPixmapHook,
)

IconEngineHookStr = Literal["available_sizes", "icon_name", "is_null", "scaled_pixmap"]


class IconEngine(QtGui.QIconEngine):
    def __repr__(self):
        return f"{type(self).__name__}()"

    def __bool__(self):
        return not self.isNull()

    def __getstate__(self):
        return bytes(self)

    def __setstate__(self, ba):
        super().__init__()
        core.DataStream.write_bytearray(ba, self)

    def __bytes__(self):
        ba = core.DataStream.create_bytearray(self)
        return bytes(ba)

    def add_file(
        self,
        path: Union[str, os.PathLike],
        size: Union[QtCore.QSize, Tuple[int, int], int],
        mode: gui.icon.ModeStr,
        state: gui.icon.StateStr,
    ):
        if mode not in gui.icon.MODE:
            raise InvalidParamError(mode, gui.icon.MODE)
        if state not in gui.icon.STATE:
            raise InvalidParamError(state, gui.icon.STATE)
        if isinstance(size, tuple):
            size = core.Size(*size)
        elif isinstance(size, int):
            size = core.Size(size, size)
        self.addFile(os.fspath(path), size, gui.icon.MODE[mode], gui.icon.STATE[state])

    def add_pixmap(
        self,
        pixmap: QtGui.QPixmap,
        mode: gui.icon.ModeStr,
        state: gui.icon.StateStr,
    ):
        if mode not in gui.icon.MODE:
            raise InvalidParamError(mode, gui.icon.MODE)
        if state not in gui.icon.STATE:
            raise InvalidParamError(state, gui.icon.STATE)
        self.addPixmap(pixmap, gui.icon.MODE[mode], gui.icon.STATE[state])

    def get_actual_size(
        self,
        size: Union[QtCore.QSize, Tuple[int, int], int],
        mode: gui.icon.ModeStr = "normal",
        state: gui.icon.StateStr = "off",
    ) -> core.Size:
        if mode not in gui.icon.MODE:
            raise InvalidParamError(mode, gui.icon.MODE)
        if state not in gui.icon.STATE:
            raise InvalidParamError(state, gui.icon.STATE)
        if isinstance(size, tuple):
            size = core.Size(*size)
        elif isinstance(size, int):
            size = core.Size(size, size)
        return core.Size(
            self.actualSize(size, gui.icon.MODE[mode], gui.icon.STATE[state])
        )

    def get_available_sizes(
        self, mode: gui.icon.ModeStr = "normal", state: gui.icon.StateStr = "off"
    ) -> List[core.Size]:
        if mode not in gui.icon.MODE:
            raise InvalidParamError(mode, gui.icon.MODE)
        if state not in gui.icon.STATE:
            raise InvalidParamError(state, gui.icon.STATE)
        return [
            core.Size(i)
            for i in self.availableSizes(gui.icon.MODE[mode], gui.icon.STATE[state])
        ]

    def get_pixmap(
        self,
        size: Union[QtCore.QSize, Tuple[int, int], int],
        mode: gui.icon.ModeStr = "normal",
        state: gui.icon.StateStr = "off",
        scale: Optional[float] = None,
    ) -> gui.Pixmap:
        if mode not in gui.icon.MODE:
            raise InvalidParamError(mode, gui.icon.MODE)
        if state not in gui.icon.STATE:
            raise InvalidParamError(state, gui.icon.STATE)
        if isinstance(size, tuple):
            size = core.Size(*size)
        elif isinstance(size, int):
            size = core.Size(size, size)
        if scale is None:
            return gui.Pixmap(
                self.pixmap(size, gui.icon.MODE[mode], gui.icon.STATE[state])
            )
        else:
            return gui.Pixmap(
                self.scaledPixmap(size, gui.icon.MODE[mode], gui.icon.STATE[state], scale)
            )


if __name__ == "__main__":
    app = gui.app()
    engine = IconEngine()
    print(repr(engine))



from __future__ import annotations

from prettyqt import core
from prettyqt.qt import QtCore


QtCore.QSignalTransition.__bases__ = (core.AbstractTransition,)


class SignalTransition(QtCore.QSignalTransition):
    pass



from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets
from prettyqt.utils import bidict


TOOLBAR_FEATURE = bidict(
    none=QtWidgets.QStyleOptionToolBar.ToolBarFeature(),
    movable=QtWidgets.QStyleOptionToolBar.Movable,
)

QtWidgets.QStyleOptionToolBar.__bases__ = (widgets.StyleOption,)


class StyleOptionToolBar(QtWidgets.QStyleOptionToolBar):
    pass



from __future__ import annotations

from prettyqt import multimedia
from prettyqt.qt import QtMultimedia


QtMultimedia.QAudioRecorder.__bases__ = (multimedia.MediaRecorder,)


class AudioRecorder(QtMultimedia.QAudioRecorder):
    pass



from __future__ import annotations

from typing import Optional, Union

from prettyqt import widgets
from prettyqt.qt import QtCore, QtHelp


QtHelp.QHelpContentWidget.__bases__ = (widgets.TreeView,)


class HelpContentWidget(QtHelp.QHelpContentWidget):
    def index_of(self, url: Union[QtCore.QUrl, str]) -> Optional[QtCore.QModelIndex]:
        if isinstance(url, str):
            url = QtCore.QUrl(url)
        idx = self.indexOf(url)
        if not idx.isValid():
            return None
        return idx



from __future__ import annotations

from prettyqt import core
from prettyqt.qt import QtHelp


class HelpContentItem(QtHelp.QHelpContentItem):
    def __len__(self):
        return self.childCount()

    def get_url(self) -> core.Url:
        return core.Url(self.url())



from __future__ import annotations

from typing import Union

from prettyqt import widgets
from prettyqt.qt import QtWidgets


QtWidgets.QSpacerItem.__bases__ = (widgets.LayoutItem,)


class SpacerItem(QtWidgets.QSpacerItem):
    def __init__(
        self,
        w: int,
        h: int,
        h_policy: Union[QtWidgets.QSizePolicy.Policy, widgets.sizepolicy.SizePolicyStr],
        v_policy: Union[QtWidgets.QSizePolicy.Policy, widgets.sizepolicy.SizePolicyStr],
    ):
        if isinstance(h_policy, QtWidgets.QSizePolicy.Policy):
            h_pol = h_policy
        else:
            h_pol = widgets.sizepolicy.SIZE_POLICY[h_policy]
        if isinstance(v_policy, QtWidgets.QSizePolicy.Policy):
            v_pol = v_policy
        else:
            v_pol = widgets.sizepolicy.SIZE_POLICY[v_policy]
        super().__init__(w, h, h_pol, v_pol)

    def change_size(
        self,
        w: int,
        h: int,
        h_policy: widgets.sizepolicy.SizePolicyStr = "minimum",
        v_policy: widgets.sizepolicy.SizePolicyStr = "minimum",
    ):
        if isinstance(h_policy, QtWidgets.QSizePolicy.Policy):
            h_pol = h_policy
        else:
            h_pol = widgets.sizepolicy.SIZE_POLICY[h_policy]
        if isinstance(v_policy, QtWidgets.QSizePolicy.Policy):
            v_pol = v_policy
        else:
            v_pol = widgets.sizepolicy.SIZE_POLICY[v_policy]
        self.changeSize(w, h, h_pol, v_pol)



from __future__ import annotations

from prettyqt import charts
from prettyqt.qt.QtCharts import QtCharts


QtCharts.QBarSeries.__bases__ = (charts.AbstractBarSeries,)


class BarSeries(QtCharts.QBarSeries):
    pass



from __future__ import annotations

import sys
import traceback
from typing import List, Literal, Optional, Union

from prettyqt import core, iconprovider, widgets
from prettyqt.qt import QtCore, QtWidgets
from prettyqt.utils import InvalidParamError, bidict, types


ICONS = bidict(
    none=QtWidgets.QMessageBox.NoIcon,
    information=QtWidgets.QMessageBox.Information,
    warning=QtWidgets.QMessageBox.Warning,
    critical=QtWidgets.QMessageBox.Critical,
    question=QtWidgets.QMessageBox.Question,
)

IconStr = Literal["none", "information", "warning", "critical", "question"]

BUTTONS = bidict(
    none=QtWidgets.QMessageBox.NoButton,
    cancel=QtWidgets.QMessageBox.Cancel,
    ok=QtWidgets.QMessageBox.Ok,
    save=QtWidgets.QMessageBox.Save,
    open=QtWidgets.QMessageBox.Open,
    close=QtWidgets.QMessageBox.Close,
    discard=QtWidgets.QMessageBox.Discard,
    apply=QtWidgets.QMessageBox.Apply,
    reset=QtWidgets.QMessageBox.Reset,
    restore_defaults=QtWidgets.QMessageBox.RestoreDefaults,
    help=QtWidgets.QMessageBox.Help,
    save_all=QtWidgets.QMessageBox.SaveAll,
    yes=QtWidgets.QMessageBox.Yes,
    yes_to_all=QtWidgets.QMessageBox.YesToAll,
    no=QtWidgets.QMessageBox.No,
    no_to_all=QtWidgets.QMessageBox.NoToAll,
    abort=QtWidgets.QMessageBox.Abort,
    retry=QtWidgets.QMessageBox.Retry,
    ignore=QtWidgets.QMessageBox.Ignore,
)

ButtonStr = Literal[
    "none",
    "cancel",
    "ok",
    "save",
    "open",
    "close",
    "discard",
    "apply",
    "reset",
    "restore_defaults",
    "help",
    "save_all",
    "yes",
    "yes_to_all",
    "no",
    "no_to_all",
    "abort",
    "retry",
    "ignore",
]

TEXT_FORMAT = bidict(
    rich=QtCore.Qt.RichText, plain=QtCore.Qt.PlainText, auto=QtCore.Qt.AutoText
)
if core.VersionNumber.get_qt_version() >= (5, 14, 0):
    TEXT_FORMAT["markdown"] = QtCore.Qt.MarkdownText

TextFormatStr = Literal["rich", "plain", "auto"]

QtWidgets.QMessageBox.__bases__ = (widgets.BaseDialog,)


class MessageBox(QtWidgets.QMessageBox):
    def __init__(
        self,
        icon: types.IconType = None,
        title: str = "",
        text: str = "",
        informative_text: str = "",
        details: str = "",
        buttons: Optional[list] = None,
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(parent)
        self.set_icon(icon)
        self.setText(text)
        self.setInformativeText(informative_text)
        self.setWindowTitle(title)
        self.setWindowFlags(
            QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint | QtCore.Qt.CustomizeWindowHint
        )
        self.setDetailedText(details)
        if isinstance(buttons, list):
            for b in buttons:
                self.add_button(b)

    @classmethod
    def message(
        cls,
        text: str,
        title: str = "",
        icon: types.IconType = None,
        detail_text: Optional[str] = None,
    ) -> str:
        m = cls("none", title, text)
        m.set_icon(icon)
        if detail_text is not None:
            m.setDetailedText(detail_text)
        return m.show_blocking()

    @classmethod
    def show_exception(cls, exception: Exception):
        exctype, value = sys.exc_info()[:2]
        tb = traceback.format_exc()
        dlg = cls(text=str(value), title=str(exctype), icon="critical", details=tb)
        dlg.show_blocking()

    def set_icon(self, icon: Union[types.IconType, IconStr]):
        if icon in ICONS:
            self.setIcon(ICONS[icon])
        else:
            ico = iconprovider.get_icon(icon)
            self.setIconPixmap(ico.get_pixmap(size=64))

    def show_blocking(self) -> ButtonStr:
        return BUTTONS.inverse[self.exec_()]

    def get_standard_buttons(self) -> List[ButtonStr]:
        return [k for k, v in BUTTONS.items() if v & self.standardButtons()]

    def add_button(self, button: ButtonStr) -> QtWidgets.QPushButton:
        """Add a default button.

        Args:
            button: button to add

        Returns:
            created button

        Raises:
            InvalidParamError: Button type not available
        """
        if button not in BUTTONS:
            raise InvalidParamError(button, BUTTONS)
        return self.addButton(BUTTONS[button])

    # @classmethod
    # def show_exception(cls, exception):
    #     header = str(exception[0])
    #     error_text = str(exception[1])
    #     widgets.MessageBox.message(error_text, header, "mdi.exclamation")

    def set_text_format(self, text_format: TextFormatStr):
        """Set the text format.

        Args:
            text_format: text format to use

        Raises:
            InvalidParamError: text format does not exist
        """
        if text_format not in TEXT_FORMAT:
            raise InvalidParamError(text_format, TEXT_FORMAT)
        self.setTextFormat(TEXT_FORMAT[text_format])

    def get_text_format(self) -> TextFormatStr:
        """Return current text format.

        Returns:
            text format
        """
        return TEXT_FORMAT.inverse[self.textFormat()]


if __name__ == "__main__":
    app = widgets.app()
    ret = MessageBox(icon="warning", title="header", text="text", details="details")
    ret.set_icon("mdi.folder")
    ret.show()
    print(ret)
    app.main_loop()



from __future__ import annotations

from prettyqt import core
from prettyqt.qt import QtCore


QtCore.QAbstractEventDispatcher.__bases__ = (core.Object,)


class AbstractEventDispatcher(QtCore.QAbstractEventDispatcher):
    pass



from __future__ import annotations

from prettyqt import core
from prettyqt.qt import QtWebChannel


QtWebChannel.QWebChannel.__bases__ = (core.Object,)


class WebChannel(QtWebChannel.QWebChannel):
    pass


if __name__ == "__main__":
    channel = WebChannel()



from __future__ import annotations

from typing import Literal

from prettyqt.qt import QtCore
from prettyqt.utils import bidict


ACCESS = bidict(
    private=QtCore.QMetaMethod.Private,
    protected=QtCore.QMetaMethod.Protected,
    public=QtCore.QMetaMethod.Public,
)

AccessStr = Literal["private", "protected", "public"]

METHOD_TYPE = bidict(
    method=QtCore.QMetaMethod.Method,
    signal=QtCore.QMetaMethod.Signal,
    slot=QtCore.QMetaMethod.Slot,
    constructor=QtCore.QMetaMethod.Constructor,
)

MethodTypeStr = Literal["method", "signal", "slot", "constructor"]


class MetaMethod:
    def __init__(self, metaobject: QtCore.QMetaMethod):
        self.item = metaobject

    def __bool__(self):
        return self.item.isValid()

    def get_access(self) -> AccessStr:
        return ACCESS.inverse[self.item.access()]

    def get_method_signature(self) -> str:
        return bytes(self.item.methodSignature()).decode()



from __future__ import annotations

from prettyqt import core, widgets
from prettyqt.qt import QtWidgets


QtWidgets.QGraphicsLineItem.__bases__ = (widgets.GraphicsItem,)


class GraphicsLineItem(QtWidgets.QGraphicsLineItem):
    def __repr__(self):
        return f"{type(self).__name__}({repr(self.get_line())})"

    def serialize_fields(self):
        return dict(line=self.get_line())

    def __setstate__(self, state):
        super().__setstate__(state)
        self.setLine(state["line"])

    def get_line(self) -> core.Line:
        return core.LineF(self.line())


if __name__ == "__main__":
    item = GraphicsLineItem(core.Point(0, 0), core.Point(2, 2))
    print(repr(item))



from __future__ import annotations

from typing import Literal

from prettyqt import gui
from prettyqt.qt import QtGui
from prettyqt.utils import InvalidParamError, bidict


STYLES = bidict(
    disc=QtGui.QTextListFormat.ListDisc,
    circle=QtGui.QTextListFormat.ListCircle,
    square=QtGui.QTextListFormat.ListSquare,
    decimal=QtGui.QTextListFormat.ListDecimal,
    lower_alpha=QtGui.QTextListFormat.ListLowerAlpha,
    upper_alpha=QtGui.QTextListFormat.ListUpperAlpha,
    lower_roman=QtGui.QTextListFormat.ListLowerRoman,
    upper_roman=QtGui.QTextListFormat.ListUpperRoman,
)

StyleStr = Literal[
    "disc",
    "circle",
    "square",
    "decimal",
    "lower_alpha",
    "upper_alpha",
    "lower_roman",
    "upper_roman",
]

QtGui.QTextListFormat.__bases__ = (gui.TextFormat,)


class TextListFormat(QtGui.QTextListFormat):
    def set_style(self, style: StyleStr):
        """Set the style.

        Args:
            style: style

        Raises:
            InvalidParamError: invalid style
        """
        if style not in STYLES:
            raise InvalidParamError(style, STYLES)
        self.setStyle(STYLES[style])

    def get_style(self) -> StyleStr:
        """Get current style.

        Returns:
            current style
        """
        return STYLES.inverse[self.style()]



from __future__ import annotations

from typing import Dict, List, Literal, Optional, Tuple, Union

from prettyqt import core, gui, iconprovider, widgets
from prettyqt.qt import QtCore, QtGui, QtWidgets
from prettyqt.utils import InvalidParamError, bidict, types


TAB_SHAPES = bidict(
    rounded=QtWidgets.QTabWidget.Rounded, triangular=QtWidgets.QTabWidget.Triangular
)

TabShapeStr = Literal["rounded", "triangular"]

TAB_POSITION = bidict(
    north=QtWidgets.QTabWidget.North,
    south=QtWidgets.QTabWidget.South,
    west=QtWidgets.QTabWidget.West,
    east=QtWidgets.QTabWidget.East,
)

TabPositionStr = Literal["north", "south", "west", "east"]

QtWidgets.QTabWidget.__bases__ = (widgets.Widget,)


class TabWidget(QtWidgets.QTabWidget):
    """Widget for managing the tabs section."""

    def __init__(
        self,
        parent: Optional[QtWidgets.QWidget] = None,
        closable: bool = False,
        detachable: bool = False,
    ) -> None:

        # Basic initalization
        super().__init__(parent)
        self.tabCloseRequested.connect(self.remove_tab)
        self.tab_bar = widgets.TabBar(self)

        self.setTabBar(self.tab_bar)

        # Used to keep a reference to detached tabs since their QMainWindow
        # does not have a parent
        self.detached_tabs: Dict[str, DetachedTab] = {}
        if detachable:
            self.set_detachable()
        self.set_closable(closable)

    def __len__(self) -> int:
        return self.count()

    def __getitem__(self, index: int) -> QtWidgets.QWidget:
        if isinstance(index, int):
            return self.widget(index)
        else:
            result = self.findChild(QtWidgets.QWidget, index)
            if result is None:
                raise KeyError("Widget not found")
            return result

    def __contains__(self, item: QtWidgets.QWidget):
        return self.indexOf(item) >= 0

    def serialize_fields(self):
        return dict(
            tabbar=self.tabBar(),
            widgets=self.get_children(),
            movable=self.isMovable(),
            document_mode=self.documentMode(),
            current_index=self.currentIndex(),
            tab_shape=self.get_tab_shape(),
            # elide_mode=self.get_elide_mode(),
            icon_size=self.iconSize(),
            tab_position=self.get_tab_position(),
        )

    def __setstate__(self, state):
        super().__setstate__(state)
        self.setTabBar(state["tabbar"])
        self.setDocumentMode(state.get("document_mode", False))
        self.setMovable(state.get("movable", False))
        self.set_tab_shape(state.get("tab_shape", "rounded"))
        self.setIconSize(state["icon_size"])
        for (widget, name, icon, tooltip, whatsthis) in state["widgets"]:
            i = self.add_tab(widget, name, icon)
            self.setTabToolTip(i, tooltip)
            self.setTabWhatsThis(i, whatsthis)
        self.setCurrentIndex(state.get("index", 0))

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def set_icon_size(self, size: Union[int, Tuple[int, int], QtCore.QSize]):
        """Set size of the icons."""
        if isinstance(size, int):
            size = core.Size(size, size)
        elif isinstance(size, tuple):
            size = core.Size(*size)
        self.setIconSize(size)

    def set_document_mode(self, state: bool = True) -> None:
        self.setDocumentMode(state)

    def set_tab_shape(self, shape: TabShapeStr):
        """Set tab shape for the tabwidget.

        Args:
            shape: tab shape to use

        Raises:
            InvalidParamError: tab shape does not exist
        """
        if shape not in TAB_SHAPES:
            raise InvalidParamError(shape, TAB_SHAPES)
        self.setTabShape(TAB_SHAPES[shape])

    def get_tab_shape(self) -> TabShapeStr:
        """Return tab shape.

        Returns:
            tab shape
        """
        return TAB_SHAPES.inverse[self.tabShape()]

    def set_tab_position(self, position: TabPositionStr):
        """Set tab position for the tabwidget.

        Args:
            position: tab position to use

        Raises:
            InvalidParamError: tab position does not exist
        """
        if position not in TAB_POSITION:
            raise InvalidParamError(position, TAB_POSITION)
        self.setTabPosition(TAB_POSITION[position])

    def get_tab_position(self) -> TabPositionStr:
        """Return tab position.

        Returns:
            tab position
        """
        return TAB_POSITION.inverse[self.tabPosition()]

    def get_children(self) -> List[tuple]:
        return [
            (
                self.widget(i),
                self.tabText(i),
                self.tab_icon(i),
                self.tabToolTip(i),
                self.tabWhatsThis(i),
            )
            for i in range(self.count())
        ]

    def tab_icon(self, i: int) -> Optional[gui.Icon]:
        icon = self.tabIcon(i)
        if icon.isNull():
            return None
        return gui.Icon(icon)

    def set_detachable(self):
        self.tab_bar.on_detach.connect(self.detach_tab)
        widgets.Application.call_on_exit(self.close_detached_tabs)
        self.setMovable(True)

    def set_closable(self, closable: bool = True):
        self.setTabsClosable(closable)

    @core.Slot(int, QtCore.QPoint)
    def detach_tab(self, index: int, point: Union[QtCore.QPoint, Tuple[int, int]]):
        """Detach tab by removing its contents and placing them in a DetachedTab window.

        Args:
            index (int): index location of the tab to be detached
            point (QtCore.QPoint): screen pos for creating the new DetachedTab window

        Returns:
            None: Description
        """
        # Get the tab content
        if isinstance(point, tuple):
            point = QtCore.QPoint(*point)
        name = self.tabText(index)
        icon = self.tab_icon(index)
        if icon is None:
            icon = self.window().windowIcon()
        widget = self.widget(index)

        try:
            widget_rect = widget.frameGeometry()
        except AttributeError:
            return

        # Create a new detached tab window
        detached_tab = DetachedTab(name, widget)
        detached_tab.set_modality("none")
        detached_tab.set_icon(icon)
        detached_tab.setGeometry(widget_rect)
        detached_tab.on_close.connect(self.attach_tab)
        detached_tab.move(point)
        detached_tab.show()

        # Create a reference to maintain access to the detached tab
        self.detached_tabs[name] = detached_tab

    def add_tab(
        self,
        item: Union[QtWidgets.QWidget, QtWidgets.QLayout],
        label: str,
        icon: types.IconType = None,
        position: Optional[int] = None,
        show: bool = False,
    ) -> int:
        if isinstance(item, QtWidgets.QLayout):
            widget = widgets.Widget()
            widget.set_layout(item)
        else:
            widget = item
        if position is None:
            position = len(self)
        if not icon:
            index = self.insertTab(position, widget, label)
        else:
            icon = iconprovider.get_icon(icon)
            index = self.insertTab(position, widget, icon, label)
        if show:
            self.setCurrentIndex(index)
        return index

    def attach_tab(
        self,
        widget: Union[QtWidgets.QWidget, QtWidgets.QLayout],
        name: str,
        icon: types.IconType = None,
        insert_at: Optional[int] = None,
    ):
        """Re-attach tab.

        Re-attach the tab by removing the content from the DetachedTab window,
        closing it, and placing the content back into the DetachableTabWidget.

        Args:
            widget (Union[QtWidgets.QWidget, QtWidgets.QLayout]): the content widget
                from the DetachedTab window
            name (str): the name of the detached tab
            icon (types.IconType, optional): the window icon for the detached tab
            insert_at (Optional[int], optional): insert the re-attached tab at the
                given index
        """
        widget.setParent(self)

        # Remove the reference
        del self.detached_tabs[name]

        # Determine if the given image and the main window icon are the same.
        # If they are, then do not add the icon to the tab
        self.add_tab(widget, name, icon=icon, position=insert_at, show=True)

    def close_detached_tabs(self):
        """Close all tabs that are currently detached."""
        tabs = list(self.detached_tabs.values())
        for tab in tabs:
            tab.close()

    @core.Slot(int)
    def remove_tab(self, index: int):
        widget = self.widget(index)
        self.removeTab(index)
        if widget is not None:
            widget.deleteLater()

    @core.Slot(QtWidgets.QWidget, str)
    def open_widget(self, widget: QtWidgets.QWidget, title: str = "Unnamed"):
        """Create a tab containing delivered widget."""
        self.add_tab(widget, title, icon="mdi.widgets", show=True)

    def set_tab(
        self, index: int, position: str, widget: Optional[QtWidgets.QWidget] = None
    ):
        self.tabBar().set_tab(index, position, widget)


class DetachedTab(widgets.MainWindow):
    """Window containing a detached tab.

    When a tab is detached, the contents are placed into this QMainWindow.
    The tab can be re-attached by closing the dialog

    Attributes:
        on_close: signal, emitted when window is closed (widget, title, icon)
    """

    on_close = core.Signal(QtWidgets.QWidget, str, QtGui.QIcon)

    def __init__(self, name: str, widget: QtWidgets.QWidget):
        super().__init__(None)

        self.set_id(name)
        self.set_title(name)

        self.widget = widget
        self.setCentralWidget(self.widget)
        self.widget.show()

    #  If the window is closed, emit the on_close and give the
    #  content widget back to the DetachableTabWidget
    def closeEvent(self, event):
        self.on_close.emit(self.widget, self.get_id(), self.windowIcon())


if __name__ == "__main__":
    app = widgets.app()
    tab_widget = TabWidget()
    widget = widgets.Widget()
    tab_widget.add_tab(widget, "Test")
    widget_2 = widgets.Widget()
    tab_widget.add_tab(widget_2, "Test 2")
    tab_widget.show()
    app.main_loop()



from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


QtWidgets.QStyleOptionFrame.__bases__ = (widgets.StyleOption,)


class StyleOptionFrame(QtWidgets.QStyleOptionFrame):
    pass



from __future__ import annotations

from typing import Optional, Type

from prettyqt import widgets
from prettyqt.qt import QtCore, QtGui, QtWidgets


TYPES = {
    bool: 1,
    int: 2,
    str: 10,
    float: 38,
    QtGui.QColor: 67,
    QtGui.QCursor: 74,
    QtCore.QDate: 14,
    QtCore.QSize: 21,
    QtCore.QTime: 15,
    list: 9,
    QtGui.QPolygon: 71,
    QtGui.QPolygonF: 86,
    QtGui.QColor: 67,
    QtGui.QColorSpace: 87,
    QtCore.QSizeF: 22,
    QtCore.QRectF: 20,
    QtCore.QLine: 23,
    QtGui.QTextLength: 77,
    dict: 8,
    QtGui.QIcon: 69,
    QtGui.QPen: 76,
    QtCore.QLineF: 24,
    QtGui.QTextFormat: 78,
    QtCore.QRect: 19,
    QtCore.QPoint: 25,
    QtCore.QUrl: 17,
    QtCore.QRegularExpression: 44,
    QtCore.QDateTime: 16,
    QtCore.QPointF: 26,
    QtGui.QPalette: 68,
    QtGui.QFont: 64,
    QtGui.QBrush: 66,
    QtGui.QRegion: 72,
    QtGui.QImage: 70,
    QtGui.QKeySequence: 75,
    QtWidgets.QSizePolicy: 121,
    QtGui.QPixmap: 65,
    QtCore.QLocale: 18,
    QtGui.QBitmap: 73,
    QtGui.QMatrix4x4: 81,
    QtGui.QVector2D: 82,
    QtGui.QVector3D: 83,
    QtGui.QVector4D: 84,
    QtGui.QQuaternion: 85,
    QtCore.QEasingCurve: 29,
    QtCore.QJsonValue: 45,
    QtCore.QJsonDocument: 48,
    QtCore.QModelIndex: 42,
    QtCore.QPersistentModelIndex: 50,
    QtCore.QUuid: 30,
    "user": 1024,
}


class ItemEditorFactory(QtWidgets.QItemEditorFactory):
    def register_editor(
        self, editor_cls: Type[QtWidgets.QWidget], typ: Optional[int] = None
    ):
        class EditorCreator(widgets.ItemEditorCreatorBase):
            def createWidget(self, parent: QtWidgets.QWidget) -> QtWidgets.QWidget:
                return editor_cls(parent=parent)

        if typ is None:
            typ = editor_cls.staticMetaObject.userProperty().userType()
        self.registerEditor(typ, EditorCreator())



from __future__ import annotations

import contextlib
from typing import Optional

from prettyqt import core
from prettyqt.qt import QtCore
from prettyqt.utils import bidict


CHECK_INDEX_OPTIONS = bidict(
    none=QtCore.QAbstractItemModel.CheckIndexOption.NoOption,
    index_is_valid=QtCore.QAbstractItemModel.CheckIndexOption.IndexIsValid,
    do_not_use_parent=QtCore.QAbstractItemModel.CheckIndexOption.DoNotUseParent,
    parent_is_invalid=QtCore.QAbstractItemModel.CheckIndexOption.ParentIsInvalid,
)

LAYOUT_CHANGE_HINT = bidict(
    none=QtCore.QAbstractItemModel.NoLayoutChangeHint,
    vertical_sort=QtCore.QAbstractItemModel.VerticalSortHint,
    horizontal_sort=QtCore.QAbstractItemModel.HorizontalSortHint,
)


QtCore.QAbstractItemModel.__bases__ = (core.Object,)


class AbstractItemModel(QtCore.QAbstractItemModel):
    def __repr__(self):
        return f"{type(self).__name__}: {self.rowCount()} rows"

    def __len__(self) -> int:
        """Return amount of rows."""
        return self.rowCount()

    # causes issues with PySide2
    # def __getitem__(self, index: Tuple[int, int]) -> QtCore.QModelIndex:
    #     return self.index(*index)

    def check_index(
        self,
        index: QtCore.QModelIndex,
        index_is_valid: bool = False,
        do_not_use_parent: bool = False,
        parent_is_invalid: bool = False,
    ) -> bool:
        flag = QtCore.QAbstractItemModel.CheckIndexOption.NoOption
        if index_is_valid:
            flag |= CHECK_INDEX_OPTIONS["index_is_valid"]  # type: ignore
        if do_not_use_parent:
            flag |= CHECK_INDEX_OPTIONS["do_not_use_parent"]  # type: ignore
        if parent_is_invalid:
            flag |= CHECK_INDEX_OPTIONS["parent_is_invalid"]  # type: ignore
        check_flag = QtCore.QAbstractItemModel.CheckIndexOptions(flag)  # type: ignore
        return self.checkIndex(index, check_flag)

    @contextlib.contextmanager
    def change_layout(self):
        """Context manager to change the layout.

        wraps calls with correct signals
        emitted at beginning: layoutAboutToBeChanged
        emitted at end: layoutChanged

        """
        self.layoutAboutToBeChanged.emit()
        yield None
        self.layoutChanged.emit()

    @contextlib.contextmanager
    def reset_model(self):
        """Context manager to reset the model.

        wraps calls with correct signals
        emitted at beginning: beginResetModel
        emitted at end: endResetModel

        """
        self.beginResetModel()
        yield None
        self.endResetModel()

    def update_row(self, row: int):
        start_index = self.index(row, 0)
        end_index = self.index(row, self.columnCount() - 1)
        self.dataChanged.emit(start_index, end_index)

    @contextlib.contextmanager
    def remove_rows(
        self,
        first: Optional[int] = None,
        last: Optional[int] = None,
        parent: Optional[QtCore.QModelIndex] = None,
    ):
        parent = QtCore.QModelIndex() if parent is None else parent
        first = first if first is not None else 0
        last = last if last is not None else self.rowCount()
        self.beginRemoveRows(parent, first, last)
        yield None
        self.endRemoveRows()

    @contextlib.contextmanager
    def remove_columns(
        self,
        first: Optional[int] = None,
        last: Optional[int] = None,
        parent: Optional[QtCore.QModelIndex] = None,
    ):
        parent = QtCore.QModelIndex() if parent is None else parent
        first = first if first is not None else 0
        last = last if last is not None else self.rowCount()
        self.beginRemoveColumns(parent, first, last)
        yield None
        self.endRemoveColumns()

    @contextlib.contextmanager
    def insert_rows(
        self,
        first: Optional[int] = None,
        last: Optional[int] = None,
        parent: Optional[QtCore.QModelIndex] = None,
    ):
        parent = QtCore.QModelIndex() if parent is None else parent
        first = first if first is not None else 0
        last = last if last is not None else self.rowCount()
        self.beginInsertRows(parent, first, last)
        yield None
        self.endInsertRows()

    @contextlib.contextmanager
    def append_rows(self, num_rows: int, parent: Optional[QtCore.QModelIndex] = None):
        parent = QtCore.QModelIndex() if parent is None else parent
        self.beginInsertRows(parent, self.rowCount(), self.rowCount() + num_rows - 1)
        yield None
        self.endInsertRows()

    @contextlib.contextmanager
    def insert_columns(
        self,
        first: Optional[int] = None,
        last: Optional[int] = None,
        parent: Optional[QtCore.QModelIndex] = None,
    ):
        parent = QtCore.QModelIndex() if parent is None else parent
        first = first if first is not None else 0
        last = last if last is not None else self.rowCount()
        self.beginInsertColumns(parent, first, last)
        yield None
        self.endInsertColumns()

    def force_reset(self):
        self.beginResetModel()
        self.endResetModel()

    def force_layoutchange(self):
        self.layoutAboutToBeChanged.emit()
        self.layoutChanged.emit()



from __future__ import annotations

from prettyqt import charts
from prettyqt.qt.QtCharts import QtCharts


QtCharts.QLogValueAxis.__bases__ = (charts.AbstractAxis,)


class LogValueAxis(QtCharts.QLogValueAxis):
    pass



from __future__ import annotations

from typing import Optional

from prettyqt import constants, core, gui
from prettyqt.qt.QtCharts import QtCharts


QtCharts.QAbstractAxis.__bases__ = (core.Object,)


class AbstractAxis(QtCharts.QAbstractAxis):
    def get_alignment(self) -> Optional[constants.SideStr]:
        """Return current alignment.

        Returns:
            alignment
        """
        alignment = self.alignment()
        if int(alignment) == 0:
            return None
        return constants.SIDES.inverse[alignment]

    def get_orientation(self) -> Optional[constants.OrientationStr]:
        """Return current orientation.

        Returns:
            orientation
        """
        orientation = self.orientation()
        if int(orientation) == 0:
            return None
        return constants.ORIENTATION.inverse[orientation]

    def get_grid_line_color(self) -> gui.Color:
        return gui.Color(self.gridLineColor())

    def get_grid_line_pen(self) -> gui.Pen:
        return gui.Pen(self.gridLinePen())

    def get_line_pen(self) -> gui.Pen:
        return gui.Pen(self.linePen())

    def get_line_pen_color(self) -> gui.Color:
        return gui.Color(self.linePenColor())

    def get_labels_color(self) -> gui.Color:
        return gui.Color(self.labelsColor())

    def get_labels_brush(self) -> gui.Brush:
        return gui.Brush(self.labelsBrush())

    def get_labels_font(self) -> gui.Font:
        return gui.Font(self.labelsFont())

    def get_title_font(self) -> gui.Font:
        return gui.Font(self.titleFont())

    def get_title_brush(self) -> gui.Brush:
        return gui.Brush(self.titleBrush())

    def get_shades_color(self) -> gui.Color:
        return gui.Color(self.shadesColor())

    def get_shades_brush(self) -> gui.Brush:
        return gui.Brush(self.shadesBrush())

    def get_shades_pen(self) -> gui.Pen:
        return gui.Pen(self.shadesPen())

    def get_shades_border_color(self) -> gui.Color:
        return gui.Color(self.shadesBorderColor())

    def get_minor_grid_line_pen(self) -> gui.Pen:
        return gui.Pen(self.minorGridLinePen())

    def get_minor_grid_line_color(self) -> gui.Color:
        return gui.Color(self.minorGridLineColor())



from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


QtWidgets.QStyleOptionSpinBox.__bases__ = (widgets.StyleOptionComplex,)


class StyleOptionSpinBox(QtWidgets.QStyleOptionSpinBox):
    pass



from __future__ import annotations

from typing import Optional, Union

from prettyqt import constants, core, widgets
from prettyqt.qt import QtCore, QtWidgets


QtWidgets.QScrollBar.__bases__ = (widgets.AbstractSlider,)


class ScrollBar(QtWidgets.QScrollBar):

    value_changed = core.Signal(int)

    def __init__(
        self,
        orientation: Union[
            QtCore.Qt.Orientation, constants.OrientationStr
        ] = "horizontal",
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        if isinstance(orientation, QtCore.Qt.Orientation):
            ori = orientation
        else:
            ori = constants.ORIENTATION[orientation]
        super().__init__(ori, parent)
        self.valueChanged.connect(self.on_value_change)



from __future__ import annotations

from typing import Optional, Pattern, Tuple, Union

import regex as re

from prettyqt import gui
from prettyqt.qt import QtCore, QtGui


PatternType = Union[str, Pattern]


class BaseRegexValidator(gui.Validator):
    def __init__(
        self, parent: Optional[QtCore.QObject] = None, regex: Optional[PatternType] = None
    ):
        super().__init__(parent)
        self.regex: Optional[Pattern] = None
        if regex:
            self.set_regex(regex)

    def __repr__(self):
        return f"{type(self).__name__}({self.get_regex()!r})"

    def __reduce__(self):
        return type(self), (self.get_regex(),)

    def __eq__(self, other: object):
        if not isinstance(other, type(self)):
            return False
        return self.regex == other.regex

    def set_regex(self, regex: PatternType):
        if isinstance(regex, str):
            self.regex = re.compile(regex)
        else:
            self.regex = regex

    def get_regex(self) -> str:
        if self.regex is None:
            raise TypeError("Validator not initialized")
        return self.regex.pattern

    def validate(  # type: ignore
        self, text: str, pos: int = 0
    ) -> Tuple[QtGui.QValidator.State, str, int]:
        if self.regex is None:
            raise TypeError("Validator not initialized")
        if text == "":
            return self.Intermediate, text, pos
        match = self.regex.match(text, partial=True)  # type: ignore
        if match is None:
            return self.Invalid, text, pos
        if match.partial:  # type: ignore
            return self.Intermediate, text, pos
        else:
            return self.Acceptable, text, pos


class IntListValidator(BaseRegexValidator):
    def __init__(
        self, allow_single: bool = True, parent: Optional[QtCore.QObject] = None
    ):
        super().__init__(parent=parent)
        self.allow_single = allow_single
        if allow_single:
            self.set_regex(r"^(\d+)(,\s*\d+)*$")
        else:
            self.set_regex(r"^[0-9][0-9\,]+[0-9]$")

    def __reduce__(self):
        return type(self), (self.allow_single,)

    def __repr__(self):
        return f"{type(self).__name__}(allow_single={self.allow_single})"


class FloatListValidator(BaseRegexValidator):
    def __init__(
        self, allow_single: bool = True, parent: Optional[QtCore.QObject] = None
    ):
        super().__init__(parent=parent)
        self.allow_single = allow_single
        if allow_single:
            self.set_regex(r"^(\s*-?\d+(\.\d+)?)(\s*,\s*-?\d+(\.\d+)?)*$")
        else:
            self.set_regex(
                r"^(\s*-?\d+(\.\d+)?)(\s*,\s*-?\d+(\.\d+)?)" r"(\s*,\s*-?\d+(\.\d+)?)*$"
            )

    def __reduce__(self):
        return type(self), (self.allow_single,)

    def __repr__(self):
        return f"{type(self).__name__}(allow_single={self.allow_single})"


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    w = widgets.LineEdit()
    val = BaseRegexValidator()
    val.set_regex(r"\w\d\d")
    w.set_validator(val)
    w.show()
    app.main_loop()



from __future__ import annotations

from typing import Optional

from prettyqt import constants, widgets
from prettyqt.qt import QtWidgets


QtWidgets.QTreeView.__bases__ = (widgets.AbstractItemView,)


class TreeView(QtWidgets.QTreeView):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        class_name = type(self).__name__
        self.set_id(class_name)
        # visual settings
        self.setAnimated(True)
        self.setRootIsDecorated(False)
        self.setAllColumnsShowFocus(True)
        self.setUniformRowHeights(True)
        self.setAlternatingRowColors(True)
        self.setWordWrap(False)

        # misc
        self.h_header = widgets.HeaderView("horizontal", parent=self)
        self.set_selection_mode("extended")

    def serialize_fields(self):
        return dict(
            all_columns_show_focus=self.allColumnsShowFocus(),
            animated=self.isAnimated(),
            auto_expand_delay=self.autoExpandDelay(),
            expands_on_double_click=self.expandsOnDoubleClick(),
            header_hidden=self.isHeaderHidden(),
            indentation=self.indentation(),
            items_expandable=self.itemsExpandable(),
            root_is_decorated=self.rootIsDecorated(),
            sorting_enabled=self.isSortingEnabled(),
            uniform_row_heights=self.uniformRowHeights(),
            word_wrap=self.wordWrap(),
        )

    @property
    def h_header(self):
        return self.header()

    @h_header.setter
    def h_header(self, header):
        self.setHeader(header)

    def expand_all(self):
        self.expandAll()

    def set_indentation(self, indentation: int):
        self.setIndentation(indentation)

    def setup_list_style(self):
        self.setSelectionBehavior(self.SelectRows)
        self.h_header.setStretchLastSection(True)

    def adapt_sizes(self):
        model = self.model()
        if model is not None and (model.rowCount() * model.columnCount()) < 1000:
            self.h_header.resizeSections(self.h_header.ResizeToContents)
        else:
            self.h_header.resize_sections("interactive")

    def sort_by_column(self, column: Optional[int], ascending: bool = True):
        column = -1 if column is None else column
        order = constants.ASCENDING if ascending else constants.DESCENDING
        self.sortByColumn(column, order)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    dlg = QtWidgets.QMainWindow()
    status_bar = TreeView()
    dlg.show()
    app.main_loop()



from __future__ import annotations

from prettyqt import quick
from prettyqt.qt import QtQuick


QtQuick.QQuickAsyncImageProvider.__bases__ = (quick.QuickImageProvider,)


class QuickAsyncImageProvider(QtQuick.QQuickAsyncImageProvider):
    pass



from __future__ import annotations

from prettyqt import multimedia
from prettyqt.qt import QtMultimediaWidgets


QtMultimediaWidgets.QVideoWidgetControl.__bases__ = (multimedia.MediaControl,)


class VideoWidgetControl(QtMultimediaWidgets.QVideoWidgetControl):
    pass



from __future__ import annotations

from prettyqt.qt import QtWidgets


class StyleOption(QtWidgets.QStyleOption):
    @classmethod
    def based_on(cls, widget: QtWidgets.QWidget) -> StyleOption:
        opt = cls()
        opt.initFrom(widget)
        return opt



from __future__ import annotations

from dataclasses import asdict, dataclass
from typing import Any, Dict, Iterator, Literal

from prettyqt.qt import QtMultimedia
from prettyqt.utils import InvalidParamError, bidict


ENDIAN = bidict(
    big_endian=QtMultimedia.QAudioFormat.BigEndian,
    little_endian=QtMultimedia.QAudioFormat.LittleEndian,
)

EndianStr = Literal["big_endian", "little_endian"]

SAMPLE_TYPES = bidict(
    unknown=QtMultimedia.QAudioFormat.Unknown,
    signed_int=QtMultimedia.QAudioFormat.SignedInt,
    unsigned_int=QtMultimedia.QAudioFormat.UnSignedInt,
    float=QtMultimedia.QAudioFormat.Float,
)

SampleTypeStr = Literal["unknown", "signed_int", "unsigned_int", "float"]


@dataclass
class Settings:
    sample_rate: int
    channel_count: int
    sample_size: int
    byte_order: EndianStr
    sample_type: SampleTypeStr
    codec: str


class AudioFormat(QtMultimedia.QAudioFormat):
    def __getitem__(self, index: str):
        return self.to_dict()[index]

    def __setitem__(self, index: str, value):
        if index == "sample_rate":
            self.setSampleRate(value)
        elif index == "channel_count":
            self.setChannelCount(value)
        elif index == "sample_size":
            self.setSampleSize(value)
        elif index == "byte_order":
            self.set_byte_order(value)
        elif index == "sample_type":
            self.set_sample_type(value)
        elif index == "codec":
            self.setCodec(value)

    def __iter__(self) -> Iterator[str]:
        return iter(self.to_dict().keys())

    def __len__(self):
        return len(self.to_dict())

    def set_sample_type(self, mode: SampleTypeStr):
        if mode not in SAMPLE_TYPES:
            raise InvalidParamError(mode, SAMPLE_TYPES)
        self.setSampleType(SAMPLE_TYPES[mode])

    def get_sample_type(self) -> SampleTypeStr:
        return SAMPLE_TYPES.inverse[self.sampleType()]

    def set_byte_order(self, order: EndianStr):
        if order not in ENDIAN:
            raise InvalidParamError(order, ENDIAN)
        self.setByteOrder(ENDIAN[order])

    def get_byte_order(self) -> EndianStr:
        return ENDIAN.inverse[self.byteOrder()]

    def to_dataclass(self) -> Settings:
        return Settings(
            sample_rate=self.sampleRate(),
            channel_count=self.channelCount(),
            sample_type=self.get_sample_type(),
            byte_order=self.get_byte_order(),
            sample_size=self.sampleSize(),
            codec=self.codec(),
        )

    def to_dict(self) -> Dict[str, Any]:
        return asdict(self.to_dataclass())

    @classmethod
    def from_dataclass(cls, data: Settings) -> AudioFormat:
        instance = cls()
        instance.setSampleRate(data.sample_rate)
        instance.setChannelCount(data.channel_count)
        instance.set_sample_type(data.sample_type)
        instance.set_byte_order(data.byte_order)
        instance.setSampleSize(data.sample_size)
        instance.setCodec(data.codec)
        return instance

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> AudioFormat:
        settings = Settings(**data)
        return cls.from_dataclass(settings)


if __name__ == "__main__":
    fmt = AudioFormat()



from __future__ import annotations

from typing import Callable, Literal, Optional, Union

from prettyqt import constants, core, gui, iconprovider
from prettyqt.qt import QtCore, QtGui, QtWidgets
from prettyqt.utils import InvalidParamError, bidict, helpers, prettyprinter, types


PRIORITIES = bidict(
    low=QtWidgets.QAction.LowPriority,
    normal=QtWidgets.QAction.NormalPriority,
    high=QtWidgets.QAction.HighPriority,
)

PriorityStr = Literal["low", "normal", "high"]

ROLES = bidict(
    none=QtWidgets.QAction.NoRole,
    text_heuristic=QtWidgets.QAction.TextHeuristicRole,
    application_specific=QtWidgets.QAction.ApplicationSpecificRole,
    about_qt=QtWidgets.QAction.AboutQtRole,
    about=QtWidgets.QAction.AboutRole,
    preferences=QtWidgets.QAction.PreferencesRole,
    quit=QtWidgets.QAction.QuitRole,
)

RoleStr = Literal[
    "none",
    "text_heuristic",
    "application_specific",
    "about_qt",
    "about",
    "preferences",
    "quit",
]

QtWidgets.QAction.__bases__ = (core.Object,)


class Action(prettyprinter.PrettyPrinter, QtWidgets.QAction):
    def __init__(
        self,
        parent: Optional[QtCore.QObject] = None,
        text: str = "",
        icon: types.IconType = None,
        shortcut: Optional[str] = None,
        tooltip: str = "",
        checkable: bool = False,
        checked: bool = False,
        statustip: str = "",
        enabled: bool = True,
        callback: Optional[Callable] = None,
    ):
        super().__init__(parent)
        self._menu = None
        self.set_text(text)
        self.set_icon(icon)
        self.set_shortcut(shortcut)
        self.set_tooltip(tooltip)
        self.set_checkable(checkable)
        self.set_checked(checked)
        self.set_statustip(statustip)
        self.set_enabled(enabled)
        if callback is not None:
            self.triggered.connect(callback)

    def __repr__(self) -> str:
        cls_name = type(self).__name__
        return f"{cls_name}({helpers.format_kwargs(self.serialize_fields())})"

    def serialize_fields(self):
        dct = dict(
            auto_repeat=self.autoRepeat(),
            text=self.text(),
            enabled=self.isEnabled(),
            visible=self.isVisible(),
            font=self.get_font(),
            shortcut=self.get_shortcut(),
            tool_tip=self.toolTip(),
            checkable=self.isCheckable(),
            checked=self.isChecked(),
            icon=self.get_icon(),
            icon_text=self.iconText(),
            priority=self.get_priority(),
            icon_visible_in_menu=self.isIconVisibleInMenu(),
            shortcut_visible_in_context_menu=self.isShortcutVisibleInContextMenu(),
            menu_role=self.get_menu_role(),
            shortcut_context=self.get_shortcut_context(),
            status_tip=self.statusTip(),
            whats_this=self.whatsThis(),
        )
        if core.VersionNumber.get_qt_version() < (6, 0, 0):
            dct["menu"] = self.menu()
        return dct

    def __setstate__(self, state):
        super().__setstate__(state)
        self.set_text(state.get("text", ""))
        self.set_enabled(state.get("enabled", True))
        self.set_shortcut(state["shortcut"])
        self.set_tooltip(state.get("tool_tip", ""))
        self.set_statustip(state.get("status_tip", ""))
        self.set_checked(state.get("checked", False))
        self.set_priority(state["priority"])
        self.set_shortcut_context(state["shortcut_context"])
        self.set_checkable(state["checkable"])
        self.setAutoRepeat(state["auto_repeat"])
        self.setVisible(state["visible"])
        self.setFont(state["font"])
        # self.setIcon(state["icon"])
        self.setIconText(state["icon_text"])
        self.setIconVisibleInMenu(state["icon_visible_in_menu"])
        self.setShortcutVisibleInContextMenu(state["shortcut_visible_in_context_menu"])
        self.set_menu_role(state["menu_role"])
        self.setWhatsThis(state["whats_this"])
        # if core.VersionNumber.get_qt_version() < (6, 0, 0):
        #     self.setMenu(state["menu"])

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def set_text(self, text: str):
        self.setText(text)

    def set_enabled(self, enabled: bool = True):
        self.setEnabled(enabled)

    def set_disabled(self):
        self.setEnabled(False)

    def set_tooltip(self, text: str):
        self.setToolTip(text)

    def set_statustip(self, text: str):
        self.setStatusTip(text)

    def set_checked(self, value: bool):
        self.setChecked(value)

    def set_checkable(self, value: bool):
        self.setCheckable(value)

    def set_icon(self, icon: types.IconType):
        """Set the icon for the action.

        Args:
            icon: icon to use
        """
        icon = iconprovider.get_icon(icon)
        self.setIcon(icon)

    def get_icon(self) -> Optional[gui.Icon]:
        icon = self.icon()
        if icon.isNull():
            return None
        return gui.Icon(icon)

    def set_shortcut(self, shortcut: Union[None, QtGui.QKeySequence, str]):
        if shortcut is None:
            shortcut = ""
        if isinstance(shortcut, str):
            shortcut = gui.KeySequence(shortcut, gui.KeySequence.PortableText)
        self.setShortcut(shortcut)

    def get_shortcut(self) -> Optional[gui.KeySequence]:
        shortcut = self.shortcut()
        if not shortcut:
            return None
        return gui.KeySequence(shortcut.toString(), gui.KeySequence.PortableText)

    def get_font(self) -> gui.Font:
        return gui.Font(self.font())

    def set_menu(self, menu):
        try:
            self.setMenu(menu)
        except AttributeError:
            self.triggered.connect(menu.exec_)
            self._menu = menu

    def menu(self):
        return self._menu

    def set_priority(self, priority: PriorityStr):
        """Set priority of the action.

        Args:
            priority: priority for the action

        Raises:
            InvalidParamError: priority does not exist
        """
        if priority not in PRIORITIES:
            raise InvalidParamError(priority, PRIORITIES)
        self.setPriority(PRIORITIES[priority])

    def get_priority(self) -> PriorityStr:
        """Return current priority.

        Returns:
            priority
        """
        return PRIORITIES.inverse[self.priority()]

    def set_shortcut_context(self, context: constants.ContextStr):
        """Set shortcut context.

        Args:
            context: shortcut context

        Raises:
            InvalidParamError: shortcut context does not exist
        """
        if context not in constants.CONTEXT:
            raise InvalidParamError(context, constants.CONTEXT)
        self.setShortcutContext(constants.CONTEXT[context])

    def get_shortcut_context(self) -> constants.ContextStr:
        """Return shortcut context.

        Returns:
            shortcut context
        """
        return constants.CONTEXT.inverse[self.shortcutContext()]

    def set_menu_role(self, role: RoleStr):
        """Set menu role.

        Args:
            role: menu role

        Raises:
            InvalidParamError: menu role does not exist
        """
        if role not in ROLES:
            raise InvalidParamError(role, ROLES)
        self.setMenuRole(ROLES[role])

    def get_menu_role(self) -> RoleStr:
        """Return menu role.

        Returns:
            menu role
        """
        return ROLES.inverse[self.menuRole()]

    def show_shortcut_in_contextmenu(self, state: bool = True):
        self.setShortcutVisibleInContextMenu(state)


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    action = Action(text="This is a test")



from __future__ import annotations

from prettyqt import core, gui
from prettyqt.qt import QtGui


QtGui.QLinearGradient.__bases__ = (gui.Gradient,)


class LinearGradient(QtGui.QLinearGradient):
    def __repr__(self):
        return f"{type(self).__name__}({self.get_start()}, {self.get_final_stop()})"

    def serialize_fields(self):
        start = self.start()
        final_stop = self.finalStop()
        return dict(start=(start[0], start[1]), final_stop=(final_stop[0], final_stop[1]))

    def get_start(self) -> core.PointF:
        return core.PointF(self.start())

    def get_final_stop(self) -> core.PointF:
        return core.PointF(self.finalStop())
